# -*- coding: utf-8 -*-
"""
Created on Tue Jul  2 11:20:21 2013

@author: micha

from initially T. Cerba

"""
import os
import sys
import wx

if wx.__version__ < "4.":
    WXPYTHON4 = False
else:
    WXPYTHON4 = True
    wx.OPEN = wx.FD_OPEN
    wx.CHANGE_DIR = wx.FD_CHANGE_DIR

    def sttip(argself, strtip):
        """ translate name of tooltip
        """
        return wx.Window.SetToolTip(argself, wx.ToolTip(strtip))

    wx.Window.SetToolTipString = sttip
from wx.lib.agw.shapedbutton import SButton

sys.path.append("..")

import FileSeries.multigrainFS as MGFS

list_txtparamPM = ["Map Summary File", "File xyz", "maptype"]

list_txtparamPM_type_dict = {"Map Summary File": str, "File xyz": str, "maptype": str}


class MainFrame_plotmaps(wx.Frame):
    def __init__(self, parent, _id, title, dict_params):
        wx.Frame.__init__(self, parent, _id, title, size=(1000, 2000))

        self.dict_params = dict_params

        self.list_of_windows = []
        self.list_txtctrl_new = []
        print((self.dict_params))
        fullpath_summaryfile = self.dict_params["Map Summary File"]
        folderpath, summaryfile = os.path.split(fullpath_summaryfile)
        fullpath_filexyz = self.dict_params["File xyz"]
        folderpathxyz, summaryfilexyz = os.path.split(fullpath_filexyz)

        font = wx.Font(18, wx.MODERN, wx.ITALIC, wx.NORMAL)

        # GUI
        self.panel = wx.Panel(self)

        txt_fileparameters = wx.StaticText(self.panel, -1, "File parameters ")
        txt_fileparameters.SetFont(font)

        txt_summary = wx.StaticText(self.panel, -1, "Map SUMMARY .dat File")
        btnbrowse1 = wx.Button(self.panel, -1, "Browse")
        btnbrowse1.Bind(wx.EVT_BUTTON, self.OnbtnBrowse_filesum)

        self.summarypath_folder = wx.TextCtrl(
            self.panel, -1, folderpath, size=(400, -1)
        )
        self.summarypath_file = wx.TextCtrl(self.panel, -1, summaryfile, size=(400, -1))

        hbox1summaryfile = wx.BoxSizer(wx.HORIZONTAL)
        hbox1summaryfile.Add(btnbrowse1, 0, wx.ALL)
        hbox1summaryfile.AddSpacer(10)
        hbox1summaryfile.Add(txt_summary, 0, wx.ALL)

        # file xyz
        txt_filexyz = wx.StaticText(self.panel, -1, "Map File XYZ .dat ")
        btnbrowse2 = wx.Button(self.panel, -1, "Browse")
        btnbrowse2.Bind(wx.EVT_BUTTON, self.OnbtnBrowse_filexyz)

        self.filexyzpath_folder = wx.TextCtrl(
            self.panel, -1, folderpathxyz, size=(400, -1)
        )
        self.filexyzpath_file = wx.TextCtrl(
            self.panel, -1, summaryfilexyz, size=(400, -1)
        )

        hbox3summaryfile = wx.BoxSizer(wx.HORIZONTAL)
        hbox3summaryfile.Add(btnbrowse2, 0, wx.ALL)
        hbox3summaryfile.AddSpacer(10)
        hbox3summaryfile.Add(txt_filexyz, 0, wx.ALL)

        txt_mapparameters = wx.StaticText(self.panel, -1, "Map Type ")
        txt_mapparameters.SetFont(font)

        self.choice_maptype = wx.Choice( self.panel, -1,
                        choices=[ "fit", "strain6_crystal", "rgb_x_sample",
                        "strain6_sample", "euler3", "stress6_crystal", "stress6_sample",
                        "res_shear_stress", "max_rss", "von_mises", "w_mrad", ], )
        self.choice_maptype.SetSelection(0)

        self.choice_maptype.Bind(wx.EVT_CHOICE, self.Onchoice_maptype)

        self.btnclearwindows = wx.Button(
            self.panel, -1, "Clear Windows", size=(200, -1)
        )
        self.btnclearwindows.Bind(wx.EVT_BUTTON, self.OnClearChildWindows)

        vbox5 = wx.BoxSizer(wx.VERTICAL)
        vbox5.Add(self.choice_maptype, 0, wx.EXPAND)
        vbox5.Add(self.btnclearwindows, 0, wx.ALL)

        btnplot = SButton(self.panel, -1, "PLOT", size=(-1, 60))
        btnplot.Bind(wx.EVT_BUTTON, self.OnPlot)

        hboxplot = wx.BoxSizer(wx.HORIZONTAL)
        hboxplot.Add(vbox5, 0, wx.ALL)
        hboxplot.Add(btnplot, 1, wx.EXPAND)

        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(txt_fileparameters, 0, wx.CENTER, 0)
        vbox.Add(hbox1summaryfile, 0, wx.ALL, 0)
        vbox.Add(self.summarypath_folder, 0, wx.ALL)
        vbox.Add(self.summarypath_file, 0, wx.ALL)
        vbox.AddSpacer(20)
        vbox.Add(hbox3summaryfile, 0, wx.ALL, 0)
        vbox.Add(self.filexyzpath_folder, 0, wx.ALL, 0)
        vbox.Add(self.filexyzpath_file, 0, wx.ALL)
        vbox.AddSpacer(20)
        vbox.Add(txt_mapparameters, 0, wx.CENTER, 0)
        vbox.Add(hboxplot, 0, wx.EXPAND, 0)

        self.panel.SetSizer(vbox)
        vbox.Fit(self)
        self.Layout()

        # tooltip
        txt_summary.SetToolTipString(
            "File elaborated by Build_Summary.py module as a summary of all images analysis"
        )
        btnbrowse1.SetToolTipString(
            "Browse to find : File elaborated by Build_Summary.py module as a summary of all images analysis"
        )
        self.summarypath_folder.SetToolTipString("folder of summary file")
        self.summarypath_file.SetToolTipString("Filename of summary file")
        txt_filexyz.SetToolTipString("A file containing 3 columns imageindex, x, y")
        self.filexyzpath_folder.SetToolTipString("folder of file xy")
        self.filexyzpath_file.SetToolTipString("Filename of file xy")

        tiptype = "Select the quantity to plot"
        txt_mapparameters.SetToolTipString(tiptype)
        self.choice_maptype.SetToolTipString(tiptype)

        self.btnclearwindows.SetToolTipString("Clear Previous plotting windows")

    def OnClearChildWindows(self, evt):
        print("killing child windows!")
        #         print "list_of_windows", self.list_of_windows
        for child in self.list_of_windows:
            try:
                child.Close()
            except:
                continue
        self.list_of_windows = []

    def OnReadParameters(self):
        self.list_txtctrl_new = []

        filesummary_path_folder = str(self.summarypath_folder.GetValue())
        filesummary_path_file = str(self.summarypath_file.GetValue())
        filesummary_path = os.path.join(filesummary_path_folder, filesummary_path_file)

        filexyz_path_folder = str(self.filexyzpath_folder.GetValue())
        filexyz_path_file = str(self.filexyzpath_file.GetValue())

        filexyz_path = os.path.join(filexyz_path_folder, filexyz_path_file)

        if filexyz_path_file in ("None",):
            filexyz_path = None

        self.dict_params["Map Summary File"] = filesummary_path
        self.dict_params["File xyz"] = filexyz_path

        self.dict_params["maptype"] = self.choice_maptype.GetString(
            self.choice_maptype.GetSelection()
        )

        return self.dict_params

    def Onchoice_maptype(self, event):
        self.dict_params["maptype"] = self.choice_maptype.GetString(
            self.choice_maptype.GetSelection()
        )

    def OnbtnBrowse_filesum(self, event):
        folder = wx.FileDialog(self, "os.path.dirname(guest)")
        if folder.ShowModal() == wx.ID_OK:
            fold, filen = os.path.split(folder.GetPath())
            self.summarypath_folder.SetValue(fold)
            self.summarypath_file.SetValue(filen)

    def OnbtnBrowse_filexyz(self, event):
        folder = wx.FileDialog(self, "os.path.dirname(guest)")
        if folder.ShowModal() == wx.ID_OK:
            fold, filen = os.path.split(folder.GetPath())
            self.filexyzpath_folder.SetValue(fold)
            self.filexyzpath_file.SetValue(filen)

    def OnPlot(self, event):
        dictpars = self.OnReadParameters()
        print(("self.dict_params", self.dict_params))

        maptype = dictpars["maptype"]
        grain_index = 0
        MGFS.plot_map_new2(self.dict_params, maptype, grain_index, App_parent=self)


class Stock_parameters_PlotMaps:
    def __init__(self, _list_txtparamPM, _list_valueparamPM):
        self.list_txtparamPM = _list_txtparamPM
        self.dict_params = _list_valueparamPM


initialparameters = {}

LaueToolsProjectFolder = os.path.dirname(os.path.abspath(os.curdir))

print(("LaueToolProjectFolder", LaueToolsProjectFolder))

MainFolder = os.path.join(LaueToolsProjectFolder, "Examples", "GeGaN")

print(("MainFolder", MainFolder))

initialparameters["IndexRefine PeakList Folder"] = os.path.join(MainFolder, "fitfiles")
initialparameters["Map Summary File"] = os.path.join(
    MainFolder, "fitfiles", "nanox2_400__SUMMARY_0_to_5_add_columns.dat"
)
initialparameters["File xyz"] = os.path.join(
    MainFolder, "fitfiles", "nanox2_400__xy_0_to_5.dat"
)
initialparameters["maptype"] = "fit"
initialparameters["filetype"] = "LT"

initialparameters["Map shape"] = (31, 41)  # (nb lines, nb images per line)

initialparameters["(stepX, stepY) microns"] = (1.0, 1.0)
MainFolder = "/home/micha/LaueProjects/LeBaudy"
initialparameters["Map Summary File"] = os.path.join(
    MainFolder, "mappos1__SUMMARY_0_to_1425_add_columns.dat"
)
initialparameters["File xyz"] = os.path.join(MainFolder, "mappos1__xy_0_to_1425.dat")

initialparameters["stiffness file"] = os.path.join(MainFolder, "si.stf")

initialparameters["nbdigits"] = 4
initialparameters["startingindex"] = 0
initialparameters["finalindex"] = 5
initialparameters["stepindex"] = 1
initialparameters["fast axis: x or y"] = "x"

# MainFolder = '/home/micha/LaueProjects/LeBaudy'
# initialparameters['Map Summary File'] = os.path.join(MainFolder,
#                                            'mappos1__SUMMARY_0_to_1425_add_columns.dat')
# initialparameters['File xyz'] = os.path.join(MainFolder, 'mappos1__xy_0_to_1425.dat')
# initialparameters['stiffness file'] = os.path.join(MainFolder, 'si.stf')


list_valueparamPM = [
    initialparameters["Map Summary File"],
    initialparameters["File xyz"],
    "fit",
]


def prepare_params_for_plot(list_val, list_key=None):
    if list_key is None:
        list_key = list_txtparamPM

    if len(list_val) != len(list_key):
        sentence = "Lengthes of list_val and list_key differ in prepare_params_for_plot() in Plot_maps\n"
        sentence += "Nb_of_parameters= %d, Nb of values %d !!!\n" % (
            len(list_key),
            len(list_val),
        )
        sentence += "Please check the number of parameters for Plot_Maps inputs"
        raise ValueError(sentence)

    zippeddict = list(zip(list_key, list_val))

    resdict = {}
    for key, val in zippeddict:
        resdict[key] = val

    return resdict


if __name__ == "__main__":

    dict_parameters = prepare_params_for_plot(list_valueparamPM)

    PlotMapsApp = wx.App()
    PMFrame = MainFrame_plotmaps(None, -1, "Plot_Maps2.py GUI Board", dict_parameters)
    PMFrame.Show(True)
    PlotMapsApp.MainLoop()



import matplotlib.pyplot as plt
import matplotlib.patches as patches
import matplotlib

import numpy as np


class LineBuilder:
    def __init__(self, line):
        self.line = line
        self.xs = list(line.get_xdata())
        self.ys = list(line.get_ydata())
        self.cid = line.figure.canvas.mpl_connect("button_press_event", self)

    def __call__(self, event):
        print("click", event)
        if event.inaxes != self.line.axes:
            return
        self.xs.append(event.xdata)
        self.ys.append(event.ydata)
        self.line.set_data(self.xs, self.ys)
        self.line.figure.canvas.draw()


class DraggablePoints(object):
    def __init__(self, artists, tolerance=5):
        for artist in artists:
            artist.set_picker(tolerance)
        self.artists = artists
        self.currently_dragging = False
        self.current_artist = None
        self.offset = (0, 0)

        for canvas in set(artist.figure.canvas for artist in self.artists):
            canvas.mpl_connect("button_press_event", self.on_press)
            canvas.mpl_connect("button_release_event", self.on_release)
            canvas.mpl_connect("pick_event", self.on_pick)
            canvas.mpl_connect("motion_notify_event", self.on_motion)

    def on_press(self, event):
        self.currently_dragging = True

    def on_release(self, event):
        self.currently_dragging = False
        self.current_artist = None

    def on_pick(self, event):
        if self.current_artist is None:
            self.current_artist = event.artist
            x0, y0 = event.artist.center
            x1, y1 = event.mouseevent.xdata, event.mouseevent.ydata
            self.offset = (x0 - x1), (y0 - y1)

    def on_motion(self, event):
        if not self.currently_dragging:
            return
        if self.current_artist is None:
            return
        dx, dy = self.offset
        self.current_artist.center = event.xdata + dx, event.ydata + dy
        self.current_artist.figure.canvas.draw()


class DraggableLine(object):
    def __init__(
        self,
        artists,
        connectingline,
        tolerance=5,
        parent=None,
        framedim=(2048, 2048),
        datatype="pixels",
    ):
        for artist in artists:
            artist.set_picker(tolerance)
        self.artists = artists
        self.currently_dragging = False
        self.current_artist = None
        self.connectingline = connectingline
        self.parent = parent
        self.framedim = framedim
        self.offset = (0, 0)

        self.datatype = datatype

        canvaslist = [artist.figure.canvas for artist in self.artists]
        #         print "available canvas", canvaslist

        for canvas in set(canvaslist):
            canvas.mpl_connect("button_press_event", self.on_press)
            canvas.mpl_connect("button_release_event", self.on_release)
            canvas.mpl_connect("pick_event", self.on_pick)
            canvas.mpl_connect("motion_notify_event", self.on_motion)
            canvas.mpl_connect("key_press_event", self.on_key_press)

    def on_key_press(self, event):
        print("evt in drag points", event)

        key = event.key

        if key in ("+", "-"):
            if key == "+":
                self.increase_line_length = True
                self.decrease_line_length = False

            elif key == "-":
                self.decrease_line_length = True
                self.increase_line_length = False

            self.change_line_length()
            return

        if key in ("p", "m"):

            if key == "p":
                self.increaseradius = 1
            elif key == "m":
                self.increaseradius = -1

            self.changecircleradius()

        else:
            return

    def on_press(self, event):
        #         print "I m pressing !"
        self.currently_dragging = True

    def on_release(self, event):
        self.currently_dragging = False
        self.current_artist = None

    def on_pick(self, event):
        #         print "picking in dragpoint"
        if self.current_artist is None:
            self.current_artist = event.artist
            if event.artist != self.connectingline:
                #                 print "touching a circle at"
                x0, y0 = event.artist.center
                #                 print 'center=', x0, y0
                x1, y1 = event.mouseevent.xdata, event.mouseevent.ydata
                self.offset = (x0 - x1), (y0 - y1)

                #                 print "self.offset", self.offset
                return True

    def on_motion(self, event):
        #         print "I m dragging in dragpoint!"
        if not self.currently_dragging:
            return
        if self.current_artist is None:
            return

        if self.datatype == "pixels":
            limited_to_positive_integers = True
            xylimits = None
        elif self.datatype == "gnomon":
            limited_to_positive_integers = False
            xylimits = (-1, 1, -1, 1)
        elif self.datatype == "2thetachi":
            # draw lines is useless
            return

        dx, dy = self.offset

        #         print 'self.offset in onMotion', self.offset

        # first extreme point
        if self.current_artist == self.artists[0]:

            if event.xdata is None:
                X0 = self.artists[0].center[0]
            else:
                X0 = event.xdata + dx
            if event.ydata is None:
                Y0 = self.artists[0].center[1]
            else:
                Y0 = event.ydata + dy

            evtx, evty = keep_in_imagearray(
                X0,
                Y0,
                self.framedim,
                limited_to_positive_integers=limited_to_positive_integers,
                xylimits=xylimits,
            )

            self.artists[0].center = evtx, evty
            #             self.current_artist.figure.canvas.draw()

            #             print 'moving pt 0'
            xs, ys = self.get_listpts_line()
            #             print "xs, ys", xs, ys
            #             print "xs0, ys0", xs[0], ys[0]

            if event is not None:
                if event.button in (1, "1"):
                    ChangeLineLength = False
                else:
                    ChangeLineLength = True

            xs[0] = self.artists[0].center[0]
            ys[0] = self.artists[0].center[1]

            if not ChangeLineLength:
                #             print "xs0, ys0", xs[0], ys[0]
                xs[1], ys[1] = center_pts([xs[0], ys[0]], [xs[2], ys[2]])

                self.artists[1].center = xs[1], ys[1]

            # middle pt constant, second point moves
            else:
                xs[2] = xs[0] + 2 * (xs[1] - xs[0])
                ys[2] = ys[0] + 2 * (ys[1] - ys[0])

                self.artists[2].center = xs[2], ys[2]

            self.connectingline.set_data(xs, ys)
            self.connectingline.figure.canvas.draw()

        # second extreme point
        if self.current_artist == self.artists[2]:

            if event.xdata is None:
                X2 = self.artists[2].center[0]
            else:
                X2 = event.xdata + dx
            if event.ydata is None:
                Y2 = self.artists[2].center[1]
            else:
                Y2 = event.ydata + dy

            evtx, evty = keep_in_imagearray(
                X2,
                Y2,
                self.framedim,
                limited_to_positive_integers=limited_to_positive_integers,
                xylimits=xylimits,
            )

            self.artists[2].center = evtx, evty

            #             print 'moving pt 2'
            xs, ys = self.get_listpts_line()
            #             print "xs, ys ", xs, ys

            if event is not None:
                if event.button in (1, "1"):
                    ChangeLineLength = False
                else:
                    ChangeLineLength = True

            xs[2] = self.artists[2].center[0]
            ys[2] = self.artists[2].center[1]

            if not ChangeLineLength:

                xs[1], ys[1] = center_pts([xs[0], ys[0]], [xs[2], ys[2]])
                self.artists[1].center = xs[1], ys[1]

            # middle pt constant
            else:
                xs[0] = xs[2] - 2 * (xs[2] - xs[1])
                ys[0] = ys[2] - 2 * (ys[2] - ys[1])

                self.artists[0].center = xs[0], ys[0]

            self.connectingline.set_data(xs, ys)
            self.connectingline.figure.canvas.draw()
        #             self.current_artist.figure.canvas.draw()

        # middle point
        if self.current_artist == self.artists[1]:

            if event.xdata is None:
                X1 = self.artists[1].center[0]
            else:
                X1 = event.xdata + dx
            if event.ydata is None:
                Y1 = self.artists[1].center[1]
            else:
                Y1 = event.ydata + dy

            xold, yold = self.artists[1].center

            evtx, evty = keep_in_imagearray(
                X1,
                Y1,
                self.framedim,
                limited_to_positive_integers=limited_to_positive_integers,
                xylimits=xylimits,
            )

            self.artists[1].center = evtx, evty

            dxc = self.artists[1].center[0] - xold
            dyc = self.artists[1].center[1] - yold

            #             print 'moving pt central'
            xs, ys = self.get_listpts_line()

            xs[1] = self.artists[1].center[0]
            ys[1] = self.artists[1].center[1]

            xs[2] = xs[2] + dxc
            ys[2] = ys[2] + dyc
            pos2 = list(self.artists[2].center)

            pos2[0] += dxc
            pos2[1] += dyc

            self.artists[2].center = pos2

            xs[0] = xs[0] + dxc
            ys[0] = ys[0] + dyc
            pos0 = list(self.artists[0].center)

            pos0[0] += dxc
            pos0[1] += dyc

            self.artists[0].center = pos0

            #             print "self.connectingline.figure.canvas", self.connectingline.figure.canvas

            self.connectingline.set_data(xs, ys)
            self.connectingline.figure.canvas.draw()

        if self.parent is not None:
            # TODO to replace by  self.parent.viewingLUTpanel.myfunction(local parameters: artist center)
            #             print "self.parent in DraggableLine", self.parent
            try:
                self.parent.viewingLUTpanel.x0, self.parent.viewingLUTpanel.y0 = self.artists[
                    0
                ].center
                self.parent.viewingLUTpanel.x2, self.parent.viewingLUTpanel.y2 = self.artists[
                    2
                ].center
                self.parent.viewingLUTpanel.updateLineProfile()
            except AttributeError:
                return

    def get_listpts_line(self):
        xs = np.array(list(self.connectingline.get_xdata()))
        ys = np.array(list(self.connectingline.get_ydata()))

        return xs, ys

    def changecircleradius(self):

        for artist in self.artists:
            artist.radius = int(artist.radius + self.increaseradius * 5.0)

    def change_line_length(self):

        print("change_line_lengths")
        xs, ys = self.get_listpts_line()

        print("xs, ys", xs, ys)

        half_length_x = 0.5 * (xs[2] - xs[0])
        half_length_y = 0.5 * (ys[2] - ys[0])

        factor = 0.5

        if self.decrease_line_length == True:
            factor = 0.5
        elif self.increase_line_length == True:
            factor = 1.5

        xs[0] = xs[1] - int(half_length_x * factor)
        ys[0] = ys[1] - int(half_length_y * factor)

        xs[2] = xs[1] + int(half_length_x * factor)
        ys[2] = ys[1] + int(half_length_y * factor)

        self.artists[0].center = xs[0], ys[0]
        self.artists[2].center = xs[2], ys[2]

        self.decrease_line_length = False
        self.increase_line_length = False

        self.connectingline.set_data(xs, ys)
        self.connectingline.figure.canvas.draw()


def keep_in_imagearray(
    x, y, framedim, limited_to_positive_integers=True, xylimits=None
):

    # for image pixel data
    if limited_to_positive_integers:
        if x < 0:
            x = 0
        if x >= framedim[1]:
            x = framedim[1] - 1

        if y < 0:
            y = 0
        if y >= framedim[0]:
            y = framedim[0] - 1
    # for other float with float x y coordinates
    else:
        if xylimits is not None:
            xmin, xmax, ymin, ymax = xylimits
        else:
            xmin, xmax, ymin, ymax = -1.0, 1.0, -1.0, 1.0

        epsilonshift = (xmax - xmin) / 200.0

        if x < xmin:
            x = xmin + epsilonshift
        if x >= xmax:
            x = xmax - epsilonshift

        if y < ymin:
            y = ymin + epsilonshift
        if y >= ymax:
            y = ymax - epsilonshift

    return x, y


def center_pts(pt1, pt2):
    return [(pt1[0] + pt2[0]) / 2.0, (pt1[1] + pt2[1]) / 2.0]


if __name__ == "__main__":
    fig, ax = plt.subplots()
    ax.set(xlim=[-1, 2], ylim=[-1, 2])

    pt1 = [0.0, 0.0]
    pt2 = [1, 1]
    ptcenter = center_pts(pt1, pt2)

    circles = [
        patches.Circle(pt1, 0.1, fc="b", alpha=0.5),
        patches.Circle(ptcenter, 0.1, fc="r", alpha=0.5),
        patches.Circle(pt2, 0.1, fc="g", alpha=0.5),
    ]

    line, = ax.plot(
        [pt1[0], ptcenter[0], pt2[0]], [pt1[1], ptcenter[1], pt2[1]], picker=1
    )
    #     linebuilder = LineBuilder(line)

    for circ in circles:
        ax.add_patch(circ)

    print("ax.patches", ax.patches)

    dr = DraggableLine(circles, line)

    plt.show()



# -*- coding: utf-8 -*-
"""
Created on Wed Jun 26 12:23:18 2013

@author: js micha

from initially T. Cerba
"""
import sys
import os

sys.path.append("..")

import wx

# this is for running through ipython
import matplotlib
matplotlib.use("WXAgg")
#------------------------

if wx.__version__ < "4.":
    WXPYTHON4 = False
else:
    WXPYTHON4 = True
    wx.OPEN = wx.FD_OPEN
    wx.CHANGE_DIR = wx.FD_CHANGE_DIR

    def sttip(argself, strtip):
        return wx.Window.SetToolTip(argself, wx.ToolTip(strtip))

    wx.Window.SetToolTipString = sttip

# LaueTools modules
if sys.version_info.major == 3:
    from .. import readmccd as RMCCD
    from .. import dict_LaueTools as DictLT
    from .. import IOLaueTools as IOLT

else:
    import readmccd as RMCCD
    import dict_LaueTools as DictLT
    import IOLaueTools as IOLT

dict_CCD = DictLT.dict_CCD
LIST_OF_CCDS = list(dict_CCD.keys())
LIST_OF_CCDS.sort()


# --- ---- Local maxima and fit parameters
LIST_TXTPARAMS = RMCCD.LIST_OPTIONS_PEAKSEARCH
# ['local_maxima_search_method',
#                 'IntensityThreshold',
#                 'thresholdConvolve',
#                 'boxsize',
#                 'PixelNearRadius',
#                 'fit_peaks_gaussian',
#                 'xtol',
#                 'FitPixelDev',
#                 'position_definition',
#                 'maxPixelDistanceRejection']

LIST_VALUESPARAMS = RMCCD.LIST_OPTIONS_VALUESPARAMS
# = [1,
#                 1000,
#                 5000,
#                 15,
#                 10,
#                 1,
#                 0.001,
#                 2.0,
#                 1,
#                 15.]

LIST_UNITSPARAMS = RMCCD.LIST_OPTIONS_TYPE_PEAKSEARCH
# ['integer flag',
#                 'integer count',
#                 'float count',
#                 'pixel',
#                 'pixel',
#                 'integer flag',
#                 '',
#                 'pixel',
#                 'integer flag',
#                 'float']


LIST_TXTPARAM_FILE_PS = [
    "ImageFolder",
    "OutputFolder",
    "ImageFilename Prefix",
    "ImageFilename Suffix",
    "ImageFilename Nbdigits",
    "Starting Image index",
    "Final Image index",
    "Image index step",
    "Background Removal",
    "BlackListed Peaks File",
    "Selected ROIs File",
    "PeakSearch Parameters File (.psp)",
]

TIP_PS = ["Folder containing image files",
    "Folder containing results Peaks List .dat files",
    "Prefix for peaks list .dat filename prefix####suffix where #### are digits of file index",
    "Image file suffix.",
    "maximum nb of digits for zero padding of filename index.(e.g. nb of # in prefix####.dat)\n0 "
    "for no zero padding.",
    "starting file index (integer)",
    "final file index (integer)",
    "incremental step for file index (integer)",
    "Background removal type: None, auto (self background) or file path to image B and optionally "
    "formula expression separated by ;\ne.g. /home/lauetools/myimageB.mccd;A-3.0*B",
    "full path to .dat file (as made by peak search procedure) containing peaks to be removed from "
    "the list of peaks found in the current image.",
    "full path to .rois file containing list of pixel centers and boxsizex, boxsizey with the "
    "simple format in each line for a roi: x y halfboxx halboxy (separated or not by , ; etc...",
    "full path to .psp file containing Peak Search parameters",
]

DICT_TOOLTIP = {}
for key, tip in zip(LIST_TXTPARAM_FILE_PS, TIP_PS):
    DICT_TOOLTIP[key] = "%s : %s" % (key, tip)


class Stock_parameters_PeakSearch:
    def __init__(self, _list_txtparamPS, _list_valueparamPS):
        self.list_txtparamIR = _list_txtparamPS
        self.list_valueparamIR = _list_valueparamPS


# --- --  Class of Local Maxima and peak fitting board
class PeakSearchParameters(wx.Frame):
    def __init__(self, parent, _id, title, _listParameters):

        _list_txtparamPS, _list_valueparamPS, _list_unitsparams = _listParameters

        self.parent = parent

        self.list_txtparamIR = _list_txtparamPS
        self.list_valueparamIR = _list_valueparamPS
        self.list_unitsparams = _list_unitsparams

        nbrows = len(_list_txtparamPS)

        wx.Frame.__init__(self, parent, _id, title, wx.DefaultPosition, wx.Size(500, nbrows * 40))

        if WXPYTHON4:
            grid = wx.FlexGridSizer(3, 10, 10)
        else:
            grid = wx.FlexGridSizer(nbrows, 3, 10, 10)

        grid.SetFlexibleDirection(wx.HORIZONTAL)
        self.panel = wx.Panel(self)

        self.tooltips()

        self.list_txtctrl = []

        # set value according to list_txtparamIR, list_valueparamIR, list_unitsparams
        for kk, elem in enumerate(self.list_txtparamIR):
            text_field = "  " + elem
            if kk == 2:
                text_field += " (method 2)"
            wxtxt = wx.StaticText(self.panel, -1, text_field)
            grid.Add(wxtxt)

            self.txtctrl = wx.TextCtrl(self.panel, -1, "", size=(100, 25))
            self.list_txtctrl.append(self.txtctrl)
            grid.Add(self.txtctrl)

            grid.Add(wx.StaticText(self.panel, -1, self.list_unitsparams[kk]))

            wxtxt.SetToolTipString(self.tips_dict[kk])
            self.txtctrl.SetToolTipString(self.tips_dict[kk])

        self.dict_param = {}
        self.setParams()

        btnSave = wx.Button(self.panel, -1, "ACCEPT && SAVE", size=(-1, 50))
        btnSave.Bind(wx.EVT_BUTTON, self.OnSave)

        btnLoad = wx.Button(self.panel, -1, "Load File", size=(-1, 50))
        btnLoad.Bind(wx.EVT_BUTTON, self.OnLoad)

        btnLoadDefault = wx.Button(self.panel, -1, "Reset", size=(-1, 50))
        btnLoadDefault.Bind(wx.EVT_BUTTON, self.OnLoadDefault)

        btnQuit = wx.Button(self.panel, -1, "Cancel && Quit", size=(-1, 50))
        btnQuit.Bind(wx.EVT_BUTTON, self.OnQuit)

        hbox = wx.BoxSizer(wx.HORIZONTAL)
        hbox.Add(btnSave, 0, wx.EXPAND)
        hbox.Add(btnLoad, 0, wx.EXPAND)
        hbox.Add(btnLoadDefault, 0, wx.EXPAND)
        hbox.Add(btnQuit, 0, wx.EXPAND)

        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(grid, 0, wx.EXPAND)
        vbox.AddSpacer(10)
        vbox.Add(hbox, 0, wx.EXPAND)

        self.panel.SetSizer(vbox, wx.EXPAND)

        # tooltip
        btnSave.SetToolTipString("Save current parameters to a .psp file")
        btnLoadDefault.SetToolTipString("Reset parameters to default values")
        btnLoad.SetToolTipString("Select a .psp file to load peak search parameters")
        btnQuit.SetToolTipString("Quit without any changes")

    def tooltips(self):
        self.tips_dict = {}
        for kk, elem in enumerate(self.list_txtparamIR):
            self.tips_dict[kk] = "%s : " % elem

        self.tips_dict[0] += "Local maxima search method to provide initial guesses parameters for spot refinement.\n"
        self.tips_dict[0] += "0 : basic threshold; 1: Array shift method 2: kernel convolution method"
        self.tips_dict[1] += "Minimum threshold level on image pixel intensity. Peaks weaker than this threshold will be omitted\n"
        self.tips_dict[1] += "For method 2: Minimum threshold level on image pixel intensity with respect to local background intensity level"
        self.tips_dict[1] += "(lowest intensity in a box centered on local maximum position)"
        self.tips_dict[2] += "Minimum threshold level on convoluted image pixel (method 2 only)"
        self.tips_dict[3] += "half boxsize of the data array (centered on local maximum pixel) to be refined"

        self.tips_dict[4] += "Minimum distance separating two local maxima"
        self.tips_dict[5] += ("Model for spot shape refinement: 0 no fit; 1 2D gaussian; 2: 2D lorentzian")
        self.tips_dict[6] += "fit parameter (useless). To be removed."

        self.tips_dict[7] += "Maximum pixel distance separating initial spot pixel position (guessed by local maxima search method) and spot shape and position refinement.\n"
        self.tips_dict[7] += "Spot whose refined position is too far from guessed one will be rejected as bad results."
        self.tips_dict[8] += "Offset convention on Spot position coordinates with respect to array indices.\n"
        self.tips_dict[8] += "0: no shift; 1: first element in array has (1,1) pixel coordinates."
        self.tips_dict[9] += "Distance between found and blacklisted peaks below which found peak will be rejected (Exclusion radius of region around each unwanted peaks)"

    def hascorrectvalue(self, kk, val):
        flag = True
        # int  = 0 1 2
        if kk in (0, 5):
            try:
                v = int(val)
                if v not in (0, 1, 2):
                    wx.MessageBox(
                        "%s must be equal to 0,1 or 2" % self.list_txtparamIR[kk],
                        "Error")
                    flag = False
            except ValueError:
                wx.MessageBox("wrong type for %s! (kk=%d) Must be integer" % (self.list_txtparamIR[kk], kk), "Error")
                flag = False

        # int  = 0 1
        if kk in (8,):
            try:
                v = int(val)
                if v not in (0, 1):
                    wx.MessageBox("%s must be equal to 0 or 1" % self.list_txtparamIR[kk], "Error")
                    flag = False
            except ValueError:
                wx.MessageBox("wrong type for %s! (kk=%d) Must be integer"
                    % (self.list_txtparamIR[kk], kk), "Error")
                flag = False

        #         elif kk == 1:
        #             try:
        #                 v = int(float(val))
        #             except ValueError:
        #                 wx.MessageBox('wrong type for %s! (kk=%d) Must be integer' % (self.list_txtparamIR[kk], kk), 'Error')
        #                 flag = False
        elif kk == 2 and val not in (None, "None"):
            try:
                v = int(float(val))
                if v <= 1:
                    wx.MessageBox("%s must be greater than 1" % self.list_txtparamIR[kk], "Error")
                    flag = False
            except ValueError:
                wx.MessageBox("wrong type for %s! (kk=%d) Must be integer"
                                % (self.list_txtparamIR[kk], kk), "Error")
                flag = False

        # positive integer > 1
        elif kk in (1, 3, 4):
            try:
                v = int(val)
                if v < 1:
                    wx.MessageBox("%s must be >= 1" % self.list_txtparamIR[kk], "Error")
                    flag = False
            except ValueError:
                wx.MessageBox("wrong type for %s! (kk=%d) Must be integer"
                                % (self.list_txtparamIR[kk], kk), "Error")
                flag = False

        # positive float
        elif kk in (6, 7, 9):
            try:
                v = float(val)
                if v <= 0:
                    wx.MessageBox("%s must be positive" % self.list_txtparamIR[kk], "Error")
                    flag = False
            except ValueError:
                wx.MessageBox("wrong type for %s! (kk=%d) Must be float"
                                % (self.list_txtparamIR[kk], kk), "Error")
                flag = False

        return flag

    def setParams(self):
        for kk, _ in enumerate(self.list_txtparamIR):
            self.list_txtctrl[kk].SetValue(str(self.list_valueparamIR[kk]))

    def getParams(self):
        self.dict_param = {}
        flag = True
        for kk, _ in enumerate(self.list_txtparamIR):
            val = self.list_txtctrl[kk].GetValue()

            if not self.hascorrectvalue(kk, val):
                flag = False
                break

            self.list_valueparamIR[kk] = val

            self.dict_param[self.list_txtparamIR[kk]] = val

        print("Get dict of Params:")
        print(self.dict_param)
        return flag

    def OnSave(self, _):

        if not self.getParams():
            return

        wcd = "PeakSearch Param.(*.psp)|*.psp|All files(*)|*"

        defaultdir = self.parent.list_txtctrl[0].GetValue()
        if not os.path.isdir(defaultdir):
            defaultdir = os.getcwd()

        file = wx.FileDialog(
                            self,
                            "Save psp File",
                            defaultDir=defaultdir,
                            wildcard=wcd,
                            style=wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
        if file.ShowModal() == wx.ID_OK:

            outputfile = file.GetPath()

            outputfilename = RMCCD.savePeakSearchConfigFile(self.dict_param, outputfilename=outputfile)

            # outputfilename has .psp extension
            self.parent.list_txtctrl[11].SetValue(outputfilename)

        self.Close()

    def OnLoad(self, _):
        pspfile = wx.FileDialog(
                                self,
                                "Open Peak Search Parameters",
                                wildcard="PeakSearch Param.(*.psp)|*.psp|All files(*)|*")
        if pspfile.ShowModal() == wx.ID_OK:

            filepsp = pspfile.GetPath()

            dict_param = RMCCD.readPeakSearchConfigFile(filepsp)

            print("dict_param", dict_param)

            for kk, key in enumerate(LIST_TXTPARAMS):
                if key in list(dict_param.keys()):
                    self.list_valueparamIR[kk] = dict_param[key]

            print("self.list_valueparamIR", self.list_valueparamIR)

            self.setParams()

    def OnLoadDefault(self, _):
        print("reset")
        self.list_valueparamIR = LIST_VALUESPARAMS
        self.setParams()

    def OnQuit(self, _):
        self.Close()


# ---  -- Class of Folder and Filename parameters for PeakSearch Series
class MainFrame_peaksearch(wx.Frame):
    def __init__(self, parent, _id, title, _initialparameters, objet_PS):
        wx.Frame.__init__(self, parent, _id, title, wx.DefaultPosition, wx.Size(1000, 500))

        self.initialparameters = _initialparameters

        self.parent = parent
        self.objet_PS = objet_PS

        self.allMaterialsnames = LIST_OF_CCDS
        self.CCDlabel = None

        if WXPYTHON4:
            grid = wx.FlexGridSizer(3, 7, 7)
        else:
            grid = wx.FlexGridSizer(11, 3, 7, 7)

        grid.SetFlexibleDirection(wx.HORIZONTAL)
        self.panel = wx.Panel(self)

        dict_tooltip = DICT_TOOLTIP
        keys_list_dicttooltip = list(DICT_TOOLTIP.keys())

        self.list_txtctrl = []

        for kk, txt_elem in enumerate(objet_PS.list_txtparamIR):
            txt = wx.StaticText(self.panel, -1, "     %s" % txt_elem)
            grid.Add(txt)

            self.txtctrl = wx.TextCtrl(self.panel, -1, "", size=(500, 25))
            self.txtctrl.SetValue(str(objet_PS.list_valueparamIR[kk]))
            self.list_txtctrl.append(self.txtctrl)
            grid.Add(self.txtctrl)

            if txt_elem in keys_list_dicttooltip:

                txt.SetToolTipString(dict_tooltip[txt_elem])
                self.txtctrl.SetToolTipString(dict_tooltip[txt_elem])

            if kk in (0, 1, 2, 8, 9, 10, 11):
                btnbrowse = wx.Button(self.panel, kk + 10, "Browse File/Folder")
                grid.Add(btnbrowse)
                if kk == 0:
                    btnbrowse.Bind(wx.EVT_BUTTON, self.OnbtnBrowse_filepath)
                    btnbrowse.SetToolTipString("Select Folder containing image files")
                elif kk == 1:
                    btnbrowse.Bind(wx.EVT_BUTTON, self.OnbtnBrowse_filepathout)
                    btnbrowse.SetToolTipString(
                        "Select Folder containing results peaks list .dat files")
                elif kk == 2:
                    btnbrowse.Bind(wx.EVT_BUTTON, self.OnbtnBrowse_CCDfileimage)
                    btnbrowse.SetToolTipString(
                        "Select one image file to get (and guess) the generic prefix of all image filenames")
                elif kk == 8:
                    btnbrowse.Bind(wx.EVT_BUTTON, self.OnbtnBrowse_bkgfile)
                    btnbrowse.SetToolTipString(
                        "Select image file to be used as background")
                elif kk == 9:
                    btnbrowse.Bind(wx.EVT_BUTTON, self.OnbtnBrowse_Blacklistfile)
                    btnbrowse.SetToolTipString(
                        "Select peak list .dat file to remove peaks from the current peaks list")
                elif kk == 10:
                    btnbrowse.Bind(wx.EVT_BUTTON, self.OnbtnBrowse_ROIslistfile)
                    btnbrowse.SetToolTipString(
                        "Select ROIs list .rois file to restrict peaksearch in theses regions")
                elif kk == 11:
                    btnbrowse.Bind(wx.EVT_BUTTON, self.OnbtnBrowse_pspfile)
                    btnbrowse.SetToolTipString("Select .psp file containing Peak Search parameters to be applied on all images")

            elif kk == 3:
                self.comboCCD = wx.ComboBox(self.panel, -1, "MARCCD165", size=(150, -1),
                                                                choices=self.allMaterialsnames,
                                                                style=wx.CB_READONLY)
                self.comboCCD.Bind(wx.EVT_COMBOBOX, self.EnterComboCCD)
                grid.Add(self.comboCCD)
            else:
                nothing = wx.StaticText(self.panel, -1, "")
                grid.Add(nothing)
            #grid.Add(wx.Button(self.panel, kk + 12, "?", size=(25, 25)))

        Createcfgbtn = wx.Button(self.panel, -1, "Edit/Create .psp file", size=(150, -1))
        Createcfgbtn.Bind(wx.EVT_BUTTON, self.OnCreatePSP)

        # multiprocessing handling
        txt_cpus = wx.StaticText(self.panel, -1, "nb CPU(s)")
        self.txtctrl_cpus = wx.TextCtrl(self.panel, -1, "1")

        # button START
        btnStart = wx.Button(self.panel, -1,
                            "START PEAK SEARCH (Peaklist files .dat in OutPutFolder)",
                            size=(300, 50))

        btnStart.Bind(wx.EVT_BUTTON, self.OnStart)

        # widgets layout--------------
        hfinal = wx.BoxSizer(wx.HORIZONTAL)
        hfinal.Add(Createcfgbtn, 0)
        hfinal.AddSpacer(10)
        hfinal.Add(txt_cpus, 0)
        hfinal.Add(self.txtctrl_cpus, 0)

        vbox = wx.BoxSizer(wx.VERTICAL)
        vbox.Add(grid, 0, wx.EXPAND)
        vbox.Add(hfinal, 0, wx.EXPAND)
        vbox.AddSpacer(5)
        vbox.Add(btnStart, 1, wx.EXPAND)

        self.panel.SetSizer(vbox)
        vbox.Fit(self)
        self.Layout()

        # tooltips
        btnStart.SetToolTipString("Start Peak Search on all images and create a list of peaks "
        "(.dat file) for each image")
        Createcfgbtn.SetToolTipString("Create .psp file containing parameters to peak search & "
        "peak position refinement")
        tipcpus = "nb of cores to use to find peaks in all images"
        txt_cpus.SetToolTipString(tipcpus)
        self.txtctrl_cpus.SetToolTipString(tipcpus)

    def OnCreatePSP(self, _):
        filepsp = self.list_txtctrl[11].GetValue()
        if not os.path.exists(filepsp):
            return

        dict_param = RMCCD.readPeakSearchConfigFile(filepsp)

        print("dict_param", dict_param)
        listvals = [] * len(LIST_VALUESPARAMS)
        for _, key in enumerate(LIST_TXTPARAMS):
            if key in list(dict_param.keys()):
                listvals.append(dict_param[key])
            else:
                print("key = %s is not in dict_param.keys()" % key)
                listvals.append(None)

        #         print "listvals", listvals

        PSPboard = PeakSearchParameters(self, -1, "PeakSearch Parameters",
                                                    (LIST_TXTPARAMS, listvals, LIST_UNITSPARAMS))
        PSPboard.Show(True)

    def OnbtnBrowse_filepath(self, _):
        folder = wx.DirDialog(self, "Select a folder containing images")
        if folder.ShowModal() == wx.ID_OK:

            abspath = folder.GetPath()

            self.list_txtctrl[0].SetValue(abspath)

            #             mainpath, lastpath = os.path.split(abspath)

            self.list_txtctrl[1].SetValue(os.path.join(abspath, "datfiles"))

    def OnbtnBrowse_filepathout(self, _):
        folder = wx.DirDialog(self, "os.path.dirname(guest)")
        if folder.ShowModal() == wx.ID_OK:

            self.list_txtctrl[1].SetValue(folder.GetPath())

    def OnbtnBrowse_CCDfileimage(self, _):
        folder = wx.FileDialog(self, "Select CCD Image file")
        if folder.ShowModal() == wx.ID_OK:

            abspath = folder.GetPath()

            filename = os.path.split(abspath)[-1]
            #             print "filename", filename
            intension, extension = filename.split(".")

            try:
                nbdigits = int(self.list_txtctrl[4].GetValue())
            except ValueError:
                nbdigits = 4

            prefixfilename = intension[:-nbdigits]
            whole_extension = "." + extension

            # special case for  blahblah_##_mar.tif case with varying number of #...
            if filename.endswith("mar.tif"):
                prefixfilename = intension.split("_")[0] + "_"
                whole_extension = "_mar.tif"
                self.list_txtctrl[4].SetValue("varying")

            self.list_txtctrl[2].SetValue(prefixfilename)
            self.list_txtctrl[3].SetValue(whole_extension)

    def EnterComboCCD(self, event):
        item = event.GetSelection()
        self.CCDlabel = self.allMaterialsnames[item]
        print("item", item)
        print("CCDlabel", self.CCDlabel)

        if self.CCDlabel == "VHR_Mar13":
            extension = str(dict_CCD[self.CCDlabel][7])
            self.list_txtctrl[4].SetValue("varying")
        else:
            extension = "." + str(dict_CCD[self.CCDlabel][7])

        self.list_txtctrl[3].SetValue(extension)

        event.Skip()

    def OnbtnBrowse_bkgfile(self, _):
        bkgfile = wx.FileDialog(self, "Select background image",
                                    wildcard="MARCCD or ROPER file (*.mccd)|*.mccd|All files(*)|*")
        if bkgfile.ShowModal() == wx.ID_OK:

            self.list_txtctrl[8].SetValue(bkgfile.GetPath())

    def OnbtnBrowse_Blacklistfile(self, _):
        blacklist_datfile = wx.FileDialog(self, "Select BlackList .dat file image",
                                            wildcard="peaklist file (*.dat)|*.dat|All files(*)|*")
        if blacklist_datfile.ShowModal() == wx.ID_OK:

            self.list_txtctrl[9].SetValue(blacklist_datfile.GetPath())

    def OnbtnBrowse_ROIslistfile(self, _):
        ROIslist_file = wx.FileDialog(self, "Select ROIs .rois file image",
                                            wildcard="ROIs file (*.rois)|*.rois|All files(*)|*")
        if ROIslist_file.ShowModal() == wx.ID_OK:

            self.list_txtctrl[10].SetValue(ROIslist_file.GetPath())

    def OnbtnBrowse_pspfile(self, _):
        pspfile = wx.FileDialog(self,
            "Select Peak Search Parameters File",
            wildcard="PeakSearch Param.(*.psp)|*.psp|All files(*)|*")
        if pspfile.ShowModal() == wx.ID_OK:

            self.list_txtctrl[11].SetValue(pspfile.GetPath())

    def Onbtnhelp_filepath(self, _):
        help_tip = "Folder containing images file"
        self.help.SetValue(str(help_tip))

    def Onbtnhelp_Nbpicture1(self, _):
        help_tip = "to be filled..."
        self.help.SetValue(str(help_tip))

    def Onbtnhelp_Nblastpicture(self, _):
        help_tip = "to be filled..."
        self.help.SetValue(str(help_tip))

    def Onbtnhelp_increment(self, _):
        help_tip = "to be filled..."
        self.help.SetValue(str(help_tip))

    def Onbtnhelp_filepathout(self, _):
        help_tip = "to be filled..."
        self.help.SetValue(str(help_tip))

    def Onbtnhelp_fileprefix(self, _):
        help_tip = "to be filled..."
        self.help.SetValue(str(help_tip))

    def Onbtnhelp_filesuffix(self, _):
        help_tip = "to be filled..."
        self.help.SetValue(str(help_tip))

    def Onbtnhelp_Nbdigits(self, _):
        help_tip = "to be filled..."
        self.help.SetValue(str(help_tip))

    def datFolderExists(self):
        datfolder = self.list_txtctrl[1].GetValue()
        if not os.path.isdir(datfolder):
            try:
                print("I have created a folder for resulting .dat files")
                print("datfolder", datfolder)
                os.mkdir(datfolder)
                return True
            except IOError:
                wx.MessageBox("Can not create %s to contain peak list .dat files !" % datfolder, "Error")
                return False

        return True

    def imagesFolderExists(self):
        imagesfolder = self.list_txtctrl[0].GetValue()

        if not os.path.isdir(imagesfolder):
            wx.MessageBox("Can not find %s containing images!" % imagesfolder, "Error")
            return False

        return True

    def OnStart(self, _):
        """   read all ctrls and start processing  """
        # read .psp file
        filepsp = self.list_txtctrl[11].GetValue()
        if not os.path.exists(filepsp):
            wx.MessageBox("Missing file ! %s"%str(filepsp), "ERROR")
            return
        print("read peak search parameters in:", filepsp)

        dict_param = RMCCD.readPeakSearchConfigFile(filepsp)

        print("Peak Search parameters", dict_param)

        self.CCDlabel = self.comboCCD.GetValue()
        print("self.CCDlabel", self.CCDlabel)

        try:
            imageindexmin = int(self.list_txtctrl[5].GetValue())
            imageindexmax = int(self.list_txtctrl[6].GetValue())
            stepimage = int(self.list_txtctrl[7].GetValue())

        except ValueError:
            wx.MessageBox("indices must be integer", "ERROR")

        if (imageindexmin >= imageindexmax or imageindexmin < 0 or imageindexmax < 0):
            wx.MessageBox("Problems with image indices. starting index should be lower "
                                                                    "than final index", "ERROR")

        fileindexrange = (imageindexmin, imageindexmax, stepimage)

        filenameprefix = self.list_txtctrl[2].GetValue()
        suffix = self.list_txtctrl[3].GetValue()

        nbdigits = self.list_txtctrl[4].GetValue()
        try:
            nbdigits_resultfiles = int(nbdigits)
        except ValueError:
            DEFAULT_DIGITSENCODING = 4
            nbdigits_resultfiles = DEFAULT_DIGITSENCODING
        if not self.datFolderExists() or not self.imagesFolderExists():
            return

        dirname_in = self.list_txtctrl[0].GetValue()
        dirname_out = self.list_txtctrl[1].GetValue()

        # setting folders for the next analysis steps

        if self.parent is not None:
            parent = self.parent  # IR

            print("parent.initialparameters", parent.initialparameters)

            parent.initialparameters["ImageFolder"] = dirname_in
            parent.initialparameters["Output Folder (Peaklist)"] = dirname_out
            parent.initialparameters["ImageFilename Prefix"] = filenameprefix
            parent.initialparameters["ImageFilename Suffix"] = suffix

            parent.initialparameters["PeakList Folder"] = dirname_out
            parent.initialparameters["IndexRefine PeakList Folder"] = os.path.join(os.path.dirname(dirname_out), "fitfiles")
            parent.initialparameters["PeakListCor Folder"] = os.path.join(os.path.dirname(dirname_out), "corfiles")
            parent.initialparameters["PeakList Filename Prefix"] = filenameprefix
            parent.initialparameters["IndexRefine Parameters File"] = os.path.join(os.path.dirname(dirname_out), "None")
            parent.initialparameters["Detector Calibration File .det"] = os.path.join(os.path.dirname(dirname_out), "None")
            parent.initialparameters["Detector Calibration File (.dat)"] = os.path.join(os.path.dirname(dirname_out), "None")
            parent.initialparameters["PeakList Filename Suffix"] = ".dat"

            parent.initialparameters["startingindex"] = imageindexmin
            parent.initialparameters["finalindex"] = imageindexmax
            parent.initialparameters["nbdigits"] = nbdigits_resultfiles
            parent.initialparameters["stepindex"] = stepimage

        #        progressMax = 100
        #        dialog = wx.ProgressDialog("A progress box", "Time remaining", progressMax,
        #        style=wx.PD_CAN_ABORT | wx.PD_ELAPSED_TIME | wx.PD_REMAINING_TIME)
        #        keepGoing = True
        #        count = 0
        #        while keepGoing and count < progressMax:
        #            count = count + 1
        #            wx.Sleep(1)
        #            keepGoing = dialog.Update(count)
        #
        #        dialog.Destroy()

        # set parameter for background removal
        background_flag = str(self.list_txtctrl[8].GetValue())
        Data_for_localMaxima, formulaexpression = RMCCD.read_background_flag(background_flag)
        dict_param["Data_for_localMaxima"] = Data_for_localMaxima
        dict_param["formulaexpression"] = formulaexpression

        # set parameter  for blacklisted peaks
        blacklistpeaklist = self.list_txtctrl[9].GetValue()
        dict_param["Remove_BlackListedPeaks_fromfile"] = RMCCD.set_blacklist_filepath(blacklistpeaklist)

        # set parameter  for blacklisted peaks
        ROIslistfile = self.list_txtctrl[10].GetValue()

        pathroisfile = RMCCD.set_rois_file(ROIslistfile)
        print('pathroisfile', pathroisfile)
        if pathroisfile is not None:
            dict_param["listrois"] = IOLT.read_roisfile(pathroisfile)
        else:
            dict_param["listrois"] = None
        if dict_param["listrois"] is not None:
            print('\n  Finding only peaks in given ROIs according to parameters of .psp file')

        try:
            nb_cpus = int(self.txtctrl_cpus.GetValue())
        except ValueError:
            wx.MessageBox("nb of cpu(s) must be positive integer!", "Error")
            return
        if nb_cpus <= 0:
            wx.MessageBox("nb of cpu(s) must be positive integer!", "Error")
            return

        # check the first imagefile to read:
        #         print "dict_param in file series", dict_param

        # dict_param['listrois']=[(723,1530,35,35),(723,1530,5,5),(723,1538,7,7),(673,1769,15,41),]
        # dict_param['IntensityThreshold']=1000
        if nb_cpus == 1:
            RMCCD.peaksearch_fileseries(fileindexrange, filenameprefix=filenameprefix,
                                                        suffix=suffix,
                                                        nbdigits=nbdigits,
                                                        dirname_in=dirname_in,
                                                        outputname=None,
                                                        dirname_out=dirname_out,
                                                        CCDLABEL=self.CCDlabel,
                                                        KF_DIRECTION="Z>0",
                                                        dictPeakSearch=dict_param,
                                                        verbose=0,
                                                        writeResultDicts=0,
                                                        computetime=1)
        else:
            RMCCD.peaksearch_multiprocessing(fileindexrange, filenameprefix=filenameprefix,
                                                            suffix=suffix,
                                                            nbdigits=nbdigits,
                                                            dirname_in=dirname_in,
                                                            outputname=None,
                                                            dirname_out=dirname_out,
                                                            CCDLABEL=self.CCDlabel,
                                                            KF_DIRECTION="Z>0",
                                                            dictPeakSearch=dict_param,
                                                            nb_of_cpu=nb_cpus,
                                                            verbose=0,
                                                            writeResultDicts=0)

def start():
    """ launcher """
    LaueToolsProjectFolder = DictLT.LAUETOOLSFOLDER
    print("LaueToolProjectFolder in main", LaueToolsProjectFolder)

    MainFolder = os.path.join(LaueToolsProjectFolder, "Examples", "GeGaN")

    print("MainFolder in m ain", MainFolder)

    initialparameters = {}
    initialparameters["ImageFolder"] = MainFolder
    initialparameters["Output Folder (Peaklist)"] = os.path.join(MainFolder, "datfiles")
    initialparameters["ImageFilename Prefix"] = "nanox2_400_"
    initialparameters["ImageFilename Suffix"] = ".mccd"
    initialparameters["PeakSearch Parameters File"] = os.path.join(
        MainFolder, "PeakSearch_nanox2_400_0000_LT_4.psp")
    initialparameters["CCDLabel"] = "MARCCD165"
    initialparameters["BackgroundRemoval"] = "auto"
    initialparameters["BlackListed Peaks File"] = None
    initialparameters["Selected ROIs File"] = None

    list_valueparamPS = [initialparameters["ImageFolder"],
                        initialparameters["Output Folder (Peaklist)"],
                        initialparameters["ImageFilename Prefix"],
                        initialparameters["ImageFilename Suffix"],
                        4, 0, 5, 1,
                        initialparameters["BackgroundRemoval"],
                        initialparameters["BlackListed Peaks File"],
                        initialparameters["Selected ROIs File"],
                        initialparameters["PeakSearch Parameters File"]]


    Stock_PS = Stock_parameters_PeakSearch(LIST_TXTPARAM_FILE_PS, list_valueparamPS)

    PeakSearchSeriesApp = wx.App()
    PeakSearchSeries = MainFrame_peaksearch(None, -1, "Peak Search Parameters Board",
                                                                        initialparameters, Stock_PS)
    PeakSearchSeries.Show()
    PeakSearchSeriesApp.MainLoop()

    # LaueToolsGUIApp = wx.App()
    # LaueToolsframe = LaueToolsGUImainframe(None, -1, "Image Viewer and PeakSearch Board",
    #                                         projectfolder=LaueToolsProjectFolder)
    # LaueToolsframe.Show()

    # MySplash(LaueToolsframe, duration=500)

    # LaueToolsGUIApp.MainLoop()

if __name__ == "__main__":

    start()



# ======================================================================
# matscipy - Python materials science tools
# https://github.com/libAtoms/matscipy
#
# Copyright (2014) James Kermode, King's College London
#                  Lars Pastewka, Karlsruhe Institute of Technology
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# ======================================================================

import itertools
import warnings

import numpy as np
from numpy.linalg import inv, norm
try:
    import scipy.stats as scipy_stats
except:
    scipy_stats = None

# import ase.units as units
# from ase.atoms import Atoms

# ##

# The indices of the full stiffness matrix of (orthorhombic) interest
Voigt_notation = [(0, 0), (1, 1), (2, 2), (1, 2), (0, 2), (0, 1)]

def full_3x3_to_Voigt_6_index(i, j):
    if i == j:
        return i
    return 6 - i - j

def Voigt_6_to_full_3x3_strain(strain_vector):
    """
    Form a 3x3 strain matrix from a 6 component vector in Voigt notation
    """
    e1, e2, e3, e4, e5, e6 = np.transpose(strain_vector)
    return np.transpose([[1.0 + e1, 0.5 * e6, 0.5 * e5],
                         [0.5 * e6, 1.0 + e2, 0.5 * e4],
                         [0.5 * e5, 0.5 * e4, 1.0 + e3]])


def Voigt_6_to_full_3x3_stress(stress_vector):
    """
    Form a 3x3 stress matrix from a 6 component vector in Voigt notation
    """
    s1, s2, s3, s4, s5, s6 = np.transpose(stress_vector)
    return np.transpose([[s1, s6, s5],
                         [s6, s2, s4],
                         [s5, s4, s3]])


def full_3x3_to_Voigt_6_strain(strain_matrix):
    """
    Form a 6 component strain vector in Voigt notation from a 3x3 matrix
    """
    strain_matrix = np.asarray(strain_matrix)
    return np.transpose([strain_matrix[..., 0, 0] - 1.0,
                         strain_matrix[..., 1, 1] - 1.0,
                         strain_matrix[..., 2, 2] - 1.0,
                         strain_matrix[..., 1, 2] + strain_matrix[..., 2, 1],
                         strain_matrix[..., 0, 2] + strain_matrix[..., 2, 0],
                         strain_matrix[..., 0, 1] + strain_matrix[..., 1, 0]])


def full_3x3_to_Voigt_6_stress(stress_matrix):
    """
    Form a 6 component stress vector in Voigt notation from a 3x3 matrix
    """
    stress_matrix = np.asarray(stress_matrix)
    return np.transpose([stress_matrix[..., 0, 0],
                         stress_matrix[..., 1, 1],
                         stress_matrix[..., 2, 2],
                         (stress_matrix[..., 1, 2] + stress_matrix[..., 1, 2]) / 2,
                         (stress_matrix[..., 0, 2] + stress_matrix[..., 0, 2]) / 2,
                         (stress_matrix[..., 0, 1] + stress_matrix[..., 0, 1]) / 2])


def Voigt_6x6_to_full_3x3x3x3(C):
    """
    Convert from the Voigt representation of the stiffness matrix to the full
    3x3x3x3 representation.

    Parameters
    ----------
    C : array_like
        6x6 stiffness matrix (Voigt notation).

    Returns
    -------
    C : array_like
        3x3x3x3 stiffness matrix.
    """

    C = np.asarray(C)
    C_out = np.zeros((3, 3, 3, 3), dtype=float)
    for i, j, k, l in itertools.product(list(range(3)), list(range(3)), list(range(3)), list(range(3))):
        Voigt_i = full_3x3_to_Voigt_6_index(i, j)
        Voigt_j = full_3x3_to_Voigt_6_index(k, l)
        C_out[i, j, k, l] = C[Voigt_i, Voigt_j]
    return C_out


def full_3x3x3x3_to_Voigt_6x6(C):
    """
    Convert from the full 3x3x3x3 representation of the stiffness matrix
    to the representation in Voigt notation. Checks symmetry in that process.
    """

    tol = 1e-3

    C = np.asarray(C)
    Voigt = np.zeros((6, 6))
    for i in range(6):
        for j in range(6):
            k, l = Voigt_notation[i]
            m, n = Voigt_notation[j]
            Voigt[i, j] = C[k, l, m, n]

            # print '---'
            # print k,l,m,n, C[k,l,m,n]
            # print m,n,k,l, C[m,n,k,l]
            # print l,k,m,n, C[l,k,m,n]
            # print k,l,n,m, C[k,l,n,m]
            # print m,n,l,k, C[m,n,l,k]
            # print n,m,k,l, C[n,m,k,l]
            # print l,k,n,m, C[l,k,n,m]
            # print n,m,l,k, C[n,m,l,k]
            # print '---'

            # Check symmetries
            assert abs(Voigt[i, j] - C[m, n, k, l]) < tol, \
                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                .format(i, j, Voigt[i, j], m, n, k, l, C[m, n, k, l])
            assert abs(Voigt[i, j] - C[l, k, m, n]) < tol, \
                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                .format(i, j, Voigt[i, j], k, l, m, n, C[l, k, m, n])
            assert abs(Voigt[i, j] - C[k, l, n, m]) < tol, \
                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                .format(i, j, Voigt[i, j], k, l, n, m, C[k, l, n, m])
            assert abs(Voigt[i, j] - C[m, n, l, k]) < tol, \
                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                .format(i, j, Voigt[i, j], m, n, l, k, C[m, n, l, k])
            assert abs(Voigt[i, j] - C[n, m, k, l]) < tol, \
                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                .format(i, j, Voigt[i, j], n, m, k, l, C[n, m, k, l])
            assert abs(Voigt[i, j] - C[l, k, n, m]) < tol, \
                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                .format(i, j, Voigt[i, j], l, k, n, m, C[l, k, n, m])
            assert abs(Voigt[i, j] - C[n, m, l, k]) < tol, \
                'Voigt[{},{}] = {}, C[{},{},{},{}] = {}' \
                .format(i, j, Voigt[i, j], n, m, l, k, C[n, m, l, k])

    return Voigt


def Voigt_6x6_to_cubic(C):
    """
    Convert the Voigt 6x6 representation into the cubic elastic constants
    C11, C12 and C44.
    """

    tol = 1e-6

    C_check = np.zeros_like(C)
    C_check[np.diag_indices_from(C_check)] = C[np.diag_indices_from(C)]
    C_check[0:3, 0:3] = C[0:3, 0:3]
    if np.any(np.abs(C - C_check) > tol):
        raise ValueError('"C" does not have cubic symmetry.')

    C11s = np.array([C[0, 0], C[1, 1], C[2, 2]])
    C12s = np.array([C[1, 2], C[0, 2], C[0, 1]])
    C44s = np.array([C[3, 3], C[4, 4], C[5, 5]])

    C11 = np.mean(C11s)
    C12 = np.mean(C12s)
    C44 = np.mean(C44s)

    if np.any(np.abs(C11 - C11s) > tol) or np.any(np.abs(C12 - C12s) > tol) or \
            np.any(np.abs(C44 - C44s) > tol):
        raise ValueError('"C" does not have cubic symmetry.')

    return np.array([C11, C12, C44])


def cubic_to_Voigt_6x6(C11, C12, C44):
    return np.array([[C11, C12, C12, 0, 0, 0],
                     [C12, C11, C12, 0, 0, 0],
                     [C12, C12, C11, 0, 0, 0],
                     [0, 0, 0, C44, 0, 0],
                     [0, 0, 0, 0, C44, 0],
                     [0, 0, 0, 0, 0, C44]])

def _invariants(s, syy=None, szz=None, syz=None, sxz=None, sxy=None,
               full_3x3_to_Voigt_6=full_3x3_to_Voigt_6_stress):
    """
    Receives a list of stress tensors and returns the three tensor invariants.
    """
    if syy is None:
        s = np.asarray(s)
        if s.shape == (6,):
            s = s.reshape(1, -1)
        elif s.shape == (3, 3):
            s = full_3x3_to_Voigt_6(s)
        elif s.shape[-1] == 3 and s.shape[-2] == 3:
            s = full_3x3_to_Voigt_6(s)
    else:
        s = np.transpose([np.transpose(s),
                          np.transpose(syy),
                          np.transpose(szz),
                          np.transpose(syz),
                          np.transpose(sxz),
                          np.transpose(sxy)])
    I1 = s[..., 0] + s[..., 1] + s[..., 2]
    I2 = s[..., 0] * s[..., 1] + s[..., 1] * s[..., 2] + s[..., 2] * s[..., 0] - s[..., 3] ** 2 - \
        s[..., 4] ** 2 - s[..., 5] ** 2
    I3 = s[..., 0] * s[..., 1] * s[..., 2] + 2 * s[..., 3] * s[..., 4] * s[..., 5] - \
        s[..., 3] ** 2 * s[..., 2] - s[..., 4] ** 2 * s[..., 0] - s[..., 5] ** 2 * s[..., 1]

    return I1, I2, I3

def invariants(s, syy=None, szz=None, syz=None, sxz=None, sxy=None,
               full_3x3_to_Voigt_6=full_3x3_to_Voigt_6_stress):
    I1, I2, I3 = _invariants(s, syy=syy, szz=szz, syz=syz, sxz=sxz, sxy=sxy,
                             full_3x3_to_Voigt_6=full_3x3_to_Voigt_6)

    J2 = I1 ** 2 / 3 - I2
    J3 = 2 * I1 ** 3 / 27 - I1 * I2 / 3 + I3

    # Return hydrostatic pressure, octahedral shear stress and J3
    return -I1 / 3, np.sqrt(2 * J2 / 3), J3

# ##

def rotate_cubic_elastic_constants(C11, C12, C44, A, tol=1e-6):
    """
    Return rotated elastic moduli for a cubic crystal given the elastic
    constant in standard C11, C12, C44 notation.

    Parameters
    ----------
    C11, C12, C44 : float
        Cubic elastic moduli.
    A : array_like
        3x3 rotation matrix.

    Returns
    -------
    C : array
        6x6 matrix of rotated elastic constants (Voigt notation).
    """

    A = np.asarray(A)

    # Is this a rotation matrix?
    if np.sometrue(np.abs(np.dot(np.array(A), np.transpose(np.array(A))) -
                          np.eye(3, dtype=float)) > tol):
        raise RuntimeError('Matrix *A* does not describe a rotation.')

    # Invariant elastic constants
    la = C12
    mu = C44
    al = C11 - la - 2 * mu

    # Construct rotated C in Voigt notation
    C = []
    for i, j in Voigt_notation:
        for k, l in Voigt_notation:
            h = 0.0
            if i == j and k == l:
                h += la
            if i == k and j == l:
                h += mu
            if i == l and j == k:
                h += mu
            h += al * np.sum(A[i, :] * A[j, :] * A[k, :] * A[l, :])
            C += [h]

    C = np.array(C)
    C.shape = (6, 6)
    return C

# ##

def rotate_elastic_constants(C, A, tol=1e-6):
    """
    Return rotated elastic moduli for a general crystal given the elastic
    constant in Voigt notation.

    Parameters
    ----------
    C : array_like
        6x6 matrix of elastic constants (Voigt notation).
    A : array_like
        3x3 rotation matrix.

    Returns
    -------
    C : array
        6x6 matrix of rotated elastic constants (Voigt notation).
    """

    A = np.asarray(A)

    # Is this a rotation matrix?
    if np.sometrue(np.abs(np.dot(np.array(A), np.transpose(np.array(A))) -
                          np.eye(3, dtype=float)) > tol):
        raise RuntimeError('Matrix *A* does not describe a rotation.')

    # Rotate
    return full_3x3x3x3_to_Voigt_6x6(np.einsum('ia,jb,kc,ld,abcd->ijkl',
                                               A, A, A, A,
                                               Voigt_6x6_to_full_3x3x3x3(C)))
# 
# # ##
# 
# class CubicElasticModuli:
#     tol = 1e-6
# 
#     def __init__(self, C11, C12, C44):
#         """
#         Initialize a cubic system with elastic constants C11, C12, C44
#         """
# 
#         warnings.warn('CubicElasticModuli is deprecated. Use '
#                       'rotate_elastic_constants function instead.')
# 
#         # la, mu, al are the three invariant elastic constants
#         self.la = C12
#         self.mu = C44
#         self.al = C11 - self.la - 2 * self.mu
# 
#         A = np.eye(3, dtype=float)
# 
#         # Compute initial stiffness matrix
#         self.rotate(A)
# 
# 
#     def rotate(self, A):
#         """
#         Compute the rotated stiffness matrix
#         """
# 
#         A = np.asarray(A)
# 
#         # Is this a rotation matrix?
#         if np.sometrue(np.abs(np.dot(np.array(A), np.transpose(np.array(A))) - 
#                               np.eye(3, dtype=float)) > self.tol):
#             raise RuntimeError('Matrix *A* does not describe a rotation.')
# 
#         C = [ ]
#         for i, j in Voigt_notation:
#             for k, l in Voigt_notation:
#                 h = 0.0
#                 if i == j and k == l:
#                     h += self.la
#                 if i == k and j == l:
#                     h += self.mu
#                 if i == l and j == k:
#                     h += self.mu
#                 h += self.al * np.sum(A[i, :] * A[j, :] * A[k, :] * A[l, :])
#                 C += [ h ]
# 
#         self.C = np.asarray(C)
#         self.C.shape = (6, 6)
#         return self.C
# 
# 
#     def _rotate_explicit(self, A):
#         """
#         Compute the rotated stiffness matrix by applying the rotation to the
#         full stiffness matrix. This function is for debugging purposes only.
#         """
# 
#         A = np.asarray(A)
# 
#         # Is this a rotation matrix?
#         if np.sometrue(np.abs(np.dot(np.array(A), np.transpose(np.array(A))) - 
#                               np.eye(3, dtype=float)) > self.tol):
#             raise RuntimeError('Matrix *A* does not describe a rotation.')
# 
#         C = np.zeros((3, 3, 3, 3), dtype=float)
# 
#         # Construct unrotated stiffness matrix
#         for i in range(3):
#             for j in range(3):
#                 for k in range(3):
#                     for m in range(3):
#                         h = 0.0
#                         if i == j and k == m:
#                             h += self.la
#                         if i == k and j == m:
#                             h += self.mu
#                         if i == m and j == k:
#                             h += self.mu
#                         if i == j and j == k and k == m:
#                             h += self.al
#                         C[i, j, k, m] = h
# 
#         # Rotate
#         C = np.einsum('ia,jb,kc,ld,abcd->ijkl', A, A, A, A, C)
# 
#         self.C = full_3x3x3x3_to_Voigt_6x6(C)
#         return self.C
# 
# 
#     def stiffness(self):
#         """
#         Return the elastic constants
#         """
# 
#         return self.C
# 
# 
#     def compliance(self):
#         """
#         Return the compliance coefficients
#         """
# 
#         return inv(self.C)
# 
# # ##
# 
# def measure_triclinic_elastic_constants(a, delta=0.001, optimizer=None,
#                                         logfile=None, **kwargs):
#     """
#     Brute-force measurement of elastic constants for a triclinic (general)
#     unit cell.
# 
#     Parameters
#     ----------
#     a : ase.Atoms
#         Atomic configuration.
#     optimizer : ase.optimizer.*
#         Optimizer to use for atomic position. Does not optimize atomic
#         position if set to None.
#     delta : float
#         Strain increment for analytical derivatives of stresses.
# 
#     Returns
#     -------
#     C : array_like
#         6x6 matrix of the elastic constants in Voigt notation.
#     """
# 
#     if optimizer is not None:
#         optimizer(a, logfile=logfile).run(**kwargs)
# 
#     r0 = a.positions.copy()
# 
#     cell = a.cell.copy()
#     volume = a.get_volume()
# 
#     C = np.zeros((3, 3, 3, 3), dtype=float)
#     for i in range(3):
#         for j in range(3):
#             a.set_cell(cell, scale_atoms=True)
#             a.set_positions(r0)
#         
#             D = np.eye(3)
#             D[i, j] += 0.5 * delta
#             D[j, i] += 0.5 * delta
#             a.set_cell(np.dot(D, cell.T).T, scale_atoms=True)
#             if optimizer is not None:
#                 optimizer(a, logfile=logfile).run(**kwargs)
#             sp = Voigt_6_to_full_3x3_stress(a.get_stress() * a.get_volume())
# 
#             D = np.eye(3)
#             D[i, j] -= 0.5 * delta
#             D[j, i] -= 0.5 * delta
#             a.set_cell(np.dot(D, cell.T).T, scale_atoms=True)
#             if optimizer is not None:
#                 optimizer(a, logfile=logfile).run(**kwargs)
#             sm = Voigt_6_to_full_3x3_stress(a.get_stress() * a.get_volume())
# 
#             C[:, :, i, j] = (sp - sm) / (2 * delta * volume)
# 
#     a.set_cell(cell, scale_atoms=True)
#     a.set_positions(r0)
# 
#     return full_3x3x3x3_to_Voigt_6x6(C)
# 
# 
# 
# Cij_symmetry = {
#    'cubic':           np.array([[1, 7, 7, 0, 0, 0],
#                                 [7, 1, 7, 0, 0, 0],
#                                 [7, 7, 1, 0, 0, 0],
#                                 [0, 0, 0, 4, 0, 0],
#                                 [0, 0, 0, 0, 4, 0],
#                                 [0, 0, 0, 0, 0, 4]]),
# 
#    'trigonal_high':   np.array([[1, 7, 8, 9, 10, 0],
#                                 [7, 1, 8, 0, -9, 0],
#                                 [8, 8, 3, 0, 0, 0],
#                                 [9, -9, 0, 4, 0, 0],
#                                 [10, 0, 0, 0, 4, 0],
#                                 [0, 0, 0, 0, 0, 6]]),
# 
#    'trigonal_low':    np.array([[1, 7, 8, 9, 10, 0 ],
#                                 [7, 1, 8, -9, -10, 0 ],
#                                 [8, 8, 3, 0, 0, 0 ],
#                                 [9, -9, 0, 4, 0, -10],
#                                 [10, -10, 0, 0, 4, 9 ],
#                                 [0, 0, 0, -10 , 9, 6 ]]),
# 
#    'tetragonal_high': np.array([[1, 7, 8, 0, 0, 0],
#                                 [7, 1, 8, 0, 0, 0],
#                                 [8, 8, 3, 0, 0, 0],
#                                 [0, 0, 0, 4, 0, 0],
#                                 [0, 0, 0, 0, 4, 0],
#                                 [0, 0, 0, 0, 0, 6]]),
# 
#    'tetragonal_low':  np.array([[1, 7, 8, 0, 0, 11],
#                                 [7, 1, 8, 0, 0, -11],
#                                 [8, 8, 3, 0, 0, 0],
#                                 [0, 0, 0, 4, 0, 0],
#                                 [0, 0, 0, 0, 4, 0],
#                                 [11, -11, 0, 0, 0, 6]]),
# 
#    'orthorhombic':    np.array([[ 1, 7, 8, 0, 0, 0],
#                                 [ 7, 2, 12, 0, 0, 0],
#                                 [ 8, 12, 3, 0, 0, 0],
#                                 [ 0, 0, 0, 4, 0, 0],
#                                 [ 0, 0, 0, 0, 5, 0],
#                                 [ 0, 0, 0, 0, 0, 6]]),
# 
#    'monoclinic':      np.array([[ 1, 7, 8, 0, 10, 0],
#                                 [ 7, 2, 12, 0, 14, 0],
#                                 [ 8, 12, 3, 0, 17, 0],
#                                 [ 0, 0, 0, 4, 0, 20],
#                                 [10, 14, 17, 0, 5, 0],
#                                 [ 0, 0, 0, 20, 0, 6]]),
#     
#     'triclinic':       np.array([[ 1, 7, 8, 9, 10, 11],
#                                  [ 7, 2, 12, 13, 14, 15],
#                                  [ 8, 12, 3, 16, 17, 18],
#                                  [ 9, 13, 16, 4, 19, 20],
#                                  [10, 14, 17, 19, 5, 21],
#                                  [11, 15, 18, 20, 21, 6 ]]),
#    }
# 
# def _dec(pattern):
#     # Decrease C_ij indices patterns by one.
#     # Used to make strain_patterns definitions below more readable.
#     return [(i - 1, j - 1) for (i, j) in pattern]
# 
# strain_patterns = {
# 
#    'cubic': [
#       # strain pattern e1+e4, yields C11, C21, C31 and C44, then C12 is average of C21 and C31
#       [ np.array([1, 0, 0, 1, 0, 0]), _dec([(1, 1), (2, 1), (3, 1), (4, 4)])]
#    ],
# 
#    'trigonal_high': [
#       # strain pattern e3 yield C13, C23 and C33
#       [ np.array([0, 0, 1, 0, 0, 0]), _dec([(1, 3), (2, 3), (3, 3)])],
# 
#       # strain pattern e1+e4 yields C11 C21 C31 and C44
#       [ np.array([1, 0, 0, 1, 0, 0]), _dec([(1, 1), (2, 1), (3, 1), (4, 4)])],
# 
#       # strain pattern e1 yields C11 C21 C31 C41 C51
#       [ np.array([1, 0, 0, 0, 0, 0]), _dec([(1, 1), (2, 1), (3, 1), (4, 1), (5, 1)])],
# 
#       # strain pattern e3+e4
#       [ np.array([0, 0, 1, 1, 0, 0]), _dec([(3, 3), (4, 4)])]
# 
#    ],
# 
#    'trigonal_low': [
#      # strain pattern e1, yields C11, C21, C31, C41, C51
#      [ np.array([1, 0, 0, 0, 0, 0]), _dec([(1, 1), (2, 1), (3, 1), (4, 1), (5, 1)])],
# 
#      # strain pattern e3 + e4, yields C33, C44
#      [ np.array([0, 0, 1, 1, 0, 0]), _dec([(3, 3), (4, 4)]) ],
# 
#      [ np.array([0, 0, 0, 0, 0, 1]), _dec([(6, 6)]) ]
#    ],
# 
#    'tetragonal': [
#      # strain pattern e1+e4
#      [ np.array([1, 0, 0, 1, 0, 0]), _dec([(1, 1), (2, 1), (3, 1), (6, 1), (4, 4)]) ],
# 
#      # strain pattern e3+e6
#      [ np.array([0, 0, 1, 0, 0, 1]), _dec([(3, 3), (6, 6)]) ]
#    ],
# 
#    'orthorhombic': [
#       # strain pattern e1+e4
#       [ np.array([1, 0, 0, 1, 0, 0]), _dec([(1, 1), (2, 1), (3, 1), (4, 4)]) ],
# 
#       # strain pattern e2+e5
#       [ np.array([0, 1, 0, 0, 1, 0]), _dec([(1, 2), (2, 2), (3, 2), (5, 5)]) ],
# 
#       # strain pattern e3+e6
#       [ np.array([0, 0, 1, 0, 0, 1]), _dec([(1, 3), (2, 3), (3, 3), (6, 6)]) ]
#    ],
# 
#    'monoclinic': [
#       # strain pattern e1+e4
#       [ np.array([1, 0, 0, 1, 0, 0]), _dec([(1, 1), (2, 1), (3, 1), (4, 4), (5, 1), (6, 4)]) ],
# 
#       # strain pattern e3+e6
#       [ np.array([0, 0, 1, 0, 0, 1]), _dec([(1, 3), (2, 3), (3, 3), (5, 3), (4, 6), (6, 6)]) ],
# 
#       # strain pattern e2
#       [ np.array([0, 1, 0, 0, 0, 0]), _dec([(1, 2), (2, 2), (3, 2), (5, 2)]) ],
# 
#       # strain pattern e5
#       [ np.array([0, 0, 0, 0, 1, 0]), _dec([(1, 5), (2, 5), (3, 5), (5, 5)]) ]
#    ],
# 
#    'triclinic': [
#       [ np.array([1, 0, 0, 0, 0, 0]), _dec([(1, 1), (2, 1), (3, 1), (4, 1), (5, 1), (6, 1)])],
#       [ np.array([0, 1, 0, 0, 0, 0]), _dec([(1, 2), (2, 2), (3, 2), (4, 2), (5, 2), (6, 2)])],
#       [ np.array([0, 0, 1, 0, 0, 0]), _dec([(1, 3), (2, 3), (3, 3), (4, 3), (5, 3), (6, 3)])],
#       [ np.array([0, 0, 0, 1, 0, 0]), _dec([(1, 4), (2, 4), (3, 4), (4, 4), (5, 4), (6, 4)])],
#       [ np.array([0, 0, 0, 0, 1, 0]), _dec([(1, 5), (2, 5), (3, 5), (4, 5), (5, 5), (6, 5)])],
#       [ np.array([0, 0, 0, 0, 0, 1]), _dec([(1, 6), (2, 6), (3, 6), (4, 6), (5, 6), (6, 6)])],
#    ]
# 
#    }
# 
# Cij_symmetry['hexagonal'] = Cij_symmetry['trigonal_high']
# Cij_symmetry[None] = Cij_symmetry['triclinic']
# 
# strain_patterns['hexagonal'] = strain_patterns['trigonal_high']
# strain_patterns['tetragonal_high'] = strain_patterns['tetragonal_low'] = strain_patterns['tetragonal']
# strain_patterns[None] = strain_patterns['triclinic']
# 
# 
# def generate_strained_configs(at0, symmetry='triclinic', N_steps=5, delta=1e-2):
#     """
#     Generate a sequence of strained configurations
# 
#     Parameters
#     ----------
#     a : ase.Atoms
#         Bulk crystal configuration - both unit cell and atomic positions
#         should be relaxed before calling this routine.
#     symmetry : string
#         Symmetry to use to determine which strain patterns are necessary.
#         Default is 'triclininc', i.e. no symmetry.
#     N_steps : int
#         Number of atomic configurations to generate for each strain pattern.
#         Default is 5. Absolute strain values range from -delta*N_steps/2
#         to +delta*N_steps/2.
#     delta : float
#         Strain increment for analytical derivatives of stresses. Default 1e-2
# 
#     Returns
#     -------
#     Generator which yields a sequence of ase.Atoms instances corresponding
#     to the minima set of strained conifugurations required to evaluate the
#     full 6x6 C_ij matrix under the assumed symmetry.
#     """
# 
#     if not symmetry in strain_patterns:
#         raise ValueError('Unknown symmetry %s. Valid options are %s' % (symmetry, strain_patterns.keys()))
# 
#     for pindex, (pattern, fit_pairs) in enumerate(strain_patterns[symmetry]):
#         for step in range(N_steps):
#             strain = np.where(pattern == 1, delta * (step + 1 - (N_steps + 1) / 2.0), 0.0)
#             at = at0.copy()
#             if at0.get_calculator() is not None:
#                 at.set_calculator(at0.get_calculator())
#             T = Voigt_6_to_full_3x3_strain(strain)
#             at.set_cell(np.dot(T, at.cell.T).T, scale_atoms=False)
#             at.positions[:] = np.dot(T, at.positions.T).T
#             at.info['strain'] = T
#             yield at
# 
# 
# # Elastic constant calculation.
# 
# # Code adapted from elastics.py script, available from
# # http://github.com/djw/elastic-constants
# #
# # Copyright (c) 2008, Dan Wilson
# # All rights reserved.
# #
# # Redistribution and use in source and binary forms, with or without
# # modification, are permitted provided that the following conditions are met:
# #     * Redistributions of source code must retain the above copyright
# #       notice, this list of conditions and the following disclaimer.
# #     * Redistributions in binary form must reproduce the above copyright
# #       notice, this list of conditions and the following disclaimer in the
# #       documentation and/or other materials provided with the distribution.
# #     * Neither the name of the copyright holder nor the
# #       names of its contributors may be used to endorse or promote products
# #       derived from this software without specific prior written permission.
# #
# # THIS SOFTWARE IS PROVIDED BY DAN WILSON ''AS IS'' AND ANY
# # EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# # WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# # DISCLAIMED. IN NO EVENT SHALL DAN WILSON BE LIABLE FOR ANY
# # DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# # (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# # LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# # ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# # (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# def fit_elastic_constants(a, symmetry='triclinic', N_steps=5, delta=1e-2, optimizer=None,
#                           verbose=True, graphics=False, logfile=None, **kwargs):
#     """
#     Compute elastic constants by linear regression of stress vs. strain
# 
#     Parameters
#     ----------
#     a : ase.Atoms or list of ase.Atoms
#         Either a single atomic configuration or a list of strained
#         configurations. If a single configuration is given, it is
#         passed :func:`generate_strained_configs` along with `symmetry`, `N_steps`,
#         and `delta` to generate the set of strained configurations.
#     symmetry : string
#         Symmetry to use to determine which strain patterns are necessary.
#         Default is 'triclininc', i.e. no symmetry.
#     N_steps : int
#         Number of atomic configurations to generate for each strain pattern.
#         Default is 5. Absolute strain values range from -delta*N_steps/2
#         to +delta*N_steps/2.    
#     delta : float
#         Strain increment for analytical derivatives of stresses.
#         Default is 1e-2.
#     optimizer : ase.optimizer.*
#         Optimizer to use for atomic positions (internal degrees of freedom)
#         for each applied strain. Initial config `a` is not optimised, and should
#         already have relaxed cell and atomic positions. Does not optimize atomic
#         positions if set to None.
#     verbose : bool
#         If True, print additional infomation about the quality of fit
#         and summarise results of C_ij and estimated errors. Default True.
#     graphics : bool
#         If True, use :mod:`matplotlib.pyplot` to plot the stress vs. strain
#         curve for each C_ij component fitted. Default True. 
#     logfile : bool
#         Log file to write optimizer output to. Default None (i.e. suppress
#         output).
#     **kwargs : dict
#         Additional arguments to pass to `optimizer.run()` method e.g. `fmax`.
# 
#     Returns
#     -------
#     C : array_like
#         6x6 matrix of the elastic constants in Voigt notation.
#     C_err : array_like
#         If scipy.stats module is available then error estimates for each C_ij
#         component are obtained from the accuracy of the linear regression.
#         Otherwise an array of np.zeros((6,6)) is returned.
#     
#     Notes
#     -----
# 
#     Code originally adapted from elastics.py script, available from
#     http://github.com/djw/elastic-constants
#     """
# 
#     if isinstance(a, Atoms):
#         # we've been passed a single Atoms object: use it to generate
#         # set of strained configurations according to symmetry
#         strained_configs = generate_strained_configs(a, symmetry, N_steps, delta)
#     else:
#         # assume we've been passed a list of strained configs
#         strained_configs = a
# 
#     if graphics:
#         import matplotlib.pyplot as plt
# 
#     def do_fit(index1, index2, stress, strain, patt):
#         if verbose:
#             print('Fitting C_%d%d' % (index1 + 1, index2 + 1))
#             print('Strain %r' % strain[:, index2])
#             print('Stress %r GPa' % (stress[:, index1] / units.GPa))
# 
#         if scipy_stats is not None:
#             cijFitted, intercept, r, tt, stderr = scipy_stats.linregress(strain[:, index2], stress[:, index1])
#         else:
#             cijFitted, intercept = np.polyfit(strain[:, index2], stress[:, index1], 1)
#             r, tt, stderr = 0.0, None, 0.0
# 
#         if verbose:
#             # print info about the fit
#             print('Cij (gradient) / GPa    :    ', cijFitted / units.GPa)
#             if scipy_stats is not None:
#                 print('Error in Cij / GPa      :    ', stderr / units.GPa)
#                 if abs(r) > 0.9:
#                     print('Correlation coefficient :    ', r)
#                 else:
#                     print('Correlation coefficient :    ', r, '     <----- WARNING')
#             else:
#                 print('scipy.stats not available, no error estimation performed')
# 
#         if graphics:
#             # position this plot in a 6x6 grid
#             sp = plt.subplot(6, 6, 6 * index1 + (index2 + 1))
#             sp.set_axis_on()
# 
#             # change the labels on the axes
#             xlabels = sp.get_xticklabels()
#             plt.setp(xlabels, 'rotation', 90, fontsize=7)
#             ylabels = sp.get_yticklabels()
#             plt.setp(ylabels, fontsize=7)
# 
#             # colour the plot depending on the strain pattern
#             colourDict = {0: '#BAD0EF', 1:'#FFCECE', 2:'#BDF4CB', 3:'#EEF093', 4:'#FFA4FF', 5:'#75ECFD'}
#             sp.set_axis_bgcolor(colourDict[patt])
# 
#             # plot the data
#             plt.plot([strain[0, index2], strain[-1, index2]],
#                      [(cijFitted * strain[0, index2] + intercept) / units.GPa,
#                       (cijFitted * strain[-1, index2] + intercept) / units.GPa])
#             plt.plot(strain[:, index2], stress[:, index1] / units.GPa, 'ro')
#             plt.xticks(strain[:, index2])
# 
#         return cijFitted, stderr
# 
#     if not symmetry in strain_patterns:
#         raise ValueError('Unknown symmetry %s. Valid options are %s' % (symmetry, strain_patterns.keys()))
# 
#     # There are 21 independent elastic constants
#     Cijs = {}
#     Cij_err = {}
# 
#     # Construct mapping from (i,j) to index into Cijs in range 1..21
#     # (upper triangle only to start with)
#     Cij_map = {}
#     Cij_map_sym = {}
#     for i in range(6):
#         for j in range(i, 6):
#             Cij_map[(i, j)] = Cij_symmetry[None][i, j]
#             Cij_map_sym[(i, j)] = Cij_symmetry[symmetry][i, j]
# 
#     # Reverse mapping, index 1..21 -> tuple (i,j) with i, j in range 0..5
#     Cij_rev_map = dict(zip(Cij_map.values(), Cij_map.keys()))
# 
#     # Add the lower triangle to Cij_map, e.g. C21 = C12
#     for (i1, i2) in Cij_map.copy().keys():
#         Cij_map[(i2, i1)] = Cij_map[(i1, i2)]
#         Cij_map_sym[(i2, i1)] = Cij_map_sym[(i1, i2)]
# 
# 
#     N_pattern = len(strain_patterns[symmetry])
#     configs = iter(strained_configs)
# 
#     strain = np.zeros((N_pattern, N_steps, 6))
#     stress = np.zeros((N_pattern, N_steps, 6))
# 
#     if graphics:
#         fig = plt.figure(num=1, figsize=(9.5, 8), facecolor='white')
#         fig.clear()
#         fig.subplots_adjust(left=0.07, right=0.97, top=0.97, bottom=0.07, wspace=0.5, hspace=0.5)
# 
#         for index1 in range(6):
#             for index2 in range(6):
#                 # position this plot in a 6x6 grid
#                 sp = plt.subplot(6, 6, 6 * (index1) + index2 + 1)
#                 sp.set_axis_off()
#                 plt.text(0.4, 0.4, "n/a")
# 
#     # Fill in strain and stress arrays from config Atoms list
#     for pattern_index, (pattern, fit_pairs) in enumerate(strain_patterns[symmetry]):
#         for step in range(N_steps):
#             at = next(configs)
#             if optimizer is not None:
#                 optimizer(at, logfile=logfile).run(**kwargs)
#             strain[pattern_index, step, :] = full_3x3_to_Voigt_6_strain(at.info['strain'])
#             stress[pattern_index, step, :] = at.get_stress()
# 
#     # Do the linear regression
#     for pattern_index, (pattern, fit_pairs) in enumerate(strain_patterns[symmetry]):
#         for (index1, index2) in fit_pairs:
#             fitted, err = do_fit(index1, index2,
#                                  stress[pattern_index, :, :],
#                                  strain[pattern_index, :, :],
#                                  pattern_index)
# 
#             index = abs(Cij_map_sym[(index1, index2)])
# 
#             if not index in Cijs:
#                 if verbose:
#                     print('Setting C%d%d (%d) to %f +/- %f' % 
#                           (index1 + 1, index2 + 1, index, fitted, err))
#                 Cijs[index] = [fitted]
#                 Cij_err[index] = [err]
#             else:
#                 if verbose:
#                     print('Updating C%d%d (%d) with value %f +/- %f' % 
#                           (index1 + 1, index2 + 1, index, fitted, err))
#                 Cijs[index].append(fitted)
#                 Cij_err[index].append(err)
#             if verbose:
#                 print('\n')
# 
# 
#     C = np.zeros((6, 6))
#     C_err = np.zeros((6, 6))
#     C_labels = np.zeros((6, 6), dtype='S4')
#     C_labels[:] = '    '
# 
#     # Convert lists to mean
#     for k in Cijs:
#         Cijs[k] = np.mean(Cijs[k])
# 
#     # Combine statistical errors
#     for k, v in Cij_err.items():
#         Cij_err[k] = np.sqrt(np.sum(np.array(v) ** 2)) / np.sqrt(len(v))
# 
#     if symmetry.startswith('trigonal'):
#         # Special case for trigonal lattice: C66 = (C11 - C12)/2
#         Cijs[Cij_map[(5, 5)]] = 0.5 * (Cijs[Cij_map[(0, 0)]] - Cijs[Cij_map[(0, 1)]])
#         Cij_err[Cij_map[(5, 5)]] = np.sqrt(Cij_err[Cij_map[(0, 0)]] ** 2 + Cij_err[Cij_map[(0, 1)]] ** 2)
# 
#     # Generate the 6x6 matrix of elastic constants
#     # - negative values signify a symmetry relation
#     for i in range(6):
#         for j in range(6):
#             index = Cij_symmetry[symmetry][i, j]
#             if index > 0:
#                 C[i, j] = Cijs[index]
#                 C_err[i, j] = Cij_err[index]
#                 ii, jj = Cij_rev_map[index]
#                 C_labels[i, j] = ' C%d%d' % (ii + 1, jj + 1)
#                 C_err[i, j] = Cij_err[index]
#             elif index < 0:
#                 C[i, j] = -Cijs[-index]
#                 C_err[i, j] = Cij_err[-index]
#                 ii, jj = Cij_rev_map[-index]
#                 C_labels[i, j] = '-C%d%d' % (ii + 1, jj + 1)
# 
#     if verbose:
#         print(np.array2string(C_labels).replace("'", ""))
#         print('\n = \n')
#         print(np.array2string(C / units.GPa,
#                               suppress_small=True,
#                               precision=2))
#         print
# 
#         # Summarise the independent components of C_ij matrix
#         printed = {}
#         for i in range(6):
#             for j in range(6):
#                 index = Cij_symmetry[symmetry][i, j]
#                 if index <= 0 or index in printed: continue
#                 print('C_%d%d = %-4.2f +/- %-4.2f GPa' % 
#                       (i + 1, j + 1, C[i, j] / units.GPa, C_err[i, j] / units.GPa))
#                 printed[index] = 1
# 
#     return C, C_err
# 
# 
# def youngs_modulus(C, l):
#     """
#     Calculate approximate Youngs modulus E_l from 6x6 elastic constants matrix C_ij
# 
#     This is the modulus for loading in the l direction. For the exact answer, taking
#     into account elastic anisotropuy, rotate the C_ij matrix to the correct frame,
#     compute the compliance matrix::
# 
#        C = ...  # 6x6 C_ij matrix in crystal frame
#        A = ...  # rotation matrix
#        Cr = rotate_elastic_constants(C, A)
#        S = np.inv(Cr)
#        E_x = 1/S[0, 0]  # Young's modulus for a pull in x direction
#        E_y = 1/S[1, 1]  # Young's modulus for a pull in y direction
#        E_z = 1/S[0, 0]  # Young's modulus for a pull in z direction
# 
#     Notes
#     -----
# 
#     Formula is from W. Brantley, Calculated elastic constants for stress problems associated 
#     with semiconductor devices. J. Appl. Phys., 44, 534 (1973).
#     """  
# 
#     S = inv(C)  # Compliance matrix
#     lhat = l / norm(l)  # Normalise directions
# 
#     # Youngs modulus in direction l, ratio of stress sigma_l 
#     # to strain response epsilon_l
#     E = 1.0 / (S[0, 0] - 2.0 * (S[0, 0] - S[0, 1] - 0.5 * S[3, 3]) * (lhat[0] * lhat[0] * lhat[1] * lhat[1] + 
#          lhat[1] * lhat[1] * lhat[2] * lhat[2] + 
#          lhat[0] * lhat[0] * lhat[2] * lhat[2]))
#     return E
# 
# 
# def poisson_ratio(C, l, m):
#     """
#     Calculate approximate Poisson ratio \nu_{lm} from 6x6 elastic constant matrix C_{ij}
# 
#     This is the response in `m` direction to pulling in `l` direction. Result is dimensionless.
# 
#     Notes
#     -----
# 
#     Formula is from W. Brantley, Calculated elastic constants for stress problems associated 
#     with semiconductor devices. J. Appl. Phys., 44, 534 (1973).
#     """
#     
#     S = inv(C)  # Compliance matrix
#     lhat = l / norm(l)  # Normalise directions
#     mhat = m / norm(m)
# 
#     # Poisson ratio v_lm: response in m direction to strain in 
#     # l direction, v_lm = - epsilon_m/epsilon_l
#     v = -((S[0, 1] + (S[0, 0] - S[0, 1] - 0.5 * S[3, 3]) * (lhat[0] * lhat[0] * mhat[0] * mhat[0] + 
#          lhat[1] * lhat[1] * mhat[1] * mhat[1] + 
#          lhat[2] * lhat[2] * mhat[2] * mhat[2])) / 
#          (S[0, 0] - 2.0 * (S[0, 0] - S[0, 1] - 0.5 * S[3, 3]) * (lhat[0] * lhat[0] * lhat[1] * lhat[1] + 
#          lhat[1] * lhat[1] * lhat[2] * lhat[2] + 
#          lhat[0] * lhat[0] * lhat[2] * lhat[2])))
#     return v
# 
# 
# def elastic_moduli(C, l=np.array([1, 0, 0]), R=None, tol=1e-6):
#     """
#     Calculate elastic moduli from 6x6 elastic constant matrix C_{ij}.
#     
#     The elastic moduli calculated are: Young's muduli, Poisson's ratios,
#     shear moduli, bulk mudulus and bulk mudulus tensor.
#     
#     If a direction l is specified, the system is rotated to have it as its
#     x direction (see Notes for details). If R is specified the system is
#     rotated according to it.
#     
#     Parameters
#     ----------
#     C : array_like
#         6x6 stiffness matrix (Voigt notation).
#     l : array_like, optional
#         3D direction vector for pull (the default is the x direction
#         of the original system)
#     R : array_like, optional
#         3x3 rotation matrix.
#     
#     Returns
#     -------
#     E : array_like
#         Young's modulus for a stress in each of the three directions
#         of the rotated system.
#     nu : array_like
#          3x3 matrix with Poisson's ratios.
#     Gm : array_like
#          3x3 matrix with shear moduli.
#     B : float
#         Bulk modulus.
#     K : array_like
#         3x3 matrix with bulk modulus tensor.
#     
#     Other Parameters
#     ----------------
#     tol : float, optional
#         tolerance for checking validity of rotation and comparison
#         of vectors.
#     
#     Notes
#     ---
#     It works by rotating the elastic constant tensor to the desired
#     direction, so it should be valid for arbitrary crystal structures.
#     If only l is specified there is an infinite number of possible
#     rotations. The chosen one is a rotation along the axis orthogonal
#     to the plane defined by the vectors (1, 0, 0) and l.
#     
#     Bulk modulus tensor as defined in
#     O. Rand and V. Rovenski, "Analytical Methods in Anisotropic
#     Elasticity", Birkh\"auser (2005), pp. 71.
#     
#     """
#     
#     if R is not None:
#         R = np.asarray(R)
# 
#         # Is this a rotation matrix?
#         if np.sometrue(np.abs(np.dot(np.array(R), np.transpose(np.array(R))) - 
#                               np.eye(3, dtype=float)) > tol):
#             raise RuntimeError('Matrix *R* does not describe a rotation.')
#     else:
#         u_a = np.array([1, 0, 0])
#         u_b = l / norm(l)  # Normalise directions
#         R = np.eye(3)
# 
#         if not np.allclose(l, u_a, rtol=tol, atol=tol):
#             u_v = np.cross(u_a, u_b)
#             u_v_mat = np.array([[ 0, -u_v[2], u_v[1]],
#                                 [ u_v[2], 0, -u_v[0]],
#                                 [-u_v[1], u_v[0], 0]])
# 
#             R = R + u_v_mat + \
#                 np.dot(u_v_mat, u_v_mat) * (1 - np.dot(u_a, u_b)) / \
#                 np.linalg.norm(u_v) ** 2
# 
#     Cr = rotate_elastic_constants(C, R)
#     S = np.linalg.inv(Cr)
# 
#     # Young's modulus for a stress in \alpha direction; \alpha = x, y, z
#     E = np.zeros(3)
#     E[0] = 1 / S[0, 0]
#     E[1] = 1 / S[1, 1]
#     E[2] = 1 / S[2, 2]
# 
#     # Poisson's ratio ($\alpha$, $\beta$); $\alpha$, $\beta$ = x, y, z
#     nu = np.array([
#             [-1, -S[1, 0] / S[0, 0], -S[2, 0] / S[0, 0]],
#             [-S[0, 1] / S[1, 1], -1, -S[2, 1] / S[1, 1]],
#             [-S[0, 2] / S[2, 2], -S[1, 2] / S[2, 2], -1]
#             ])
# 
#     # Shear modulus ($\alpha$, $\beta$); $\alpha$, $\beta$ = x, y, z
#     G = np.zeros(3)
#     G[0] = 1 / S[3, 3]  # Shear modulus yz
#     G[1] = 1 / S[4, 4]  # Shear modulus zx
#     G[2] = 1 / S[5, 5]  # Shear modulus xy
#     Gm = np.array([
#             [E[0] / 4, G[2], G[1]],
#             [G[2], E[1] / 4, G[0]],
#             [G[1], G[0], E[2] / 4]
#             ])
# 
#     # Bulk modulus
#     B = 1 / np.sum(S[0:3, 0:3])
#     
#     # Bulk modulus tensor
#     Crt = Voigt_6x6_to_full_3x3x3x3(Cr)
#     K = np.einsum('ijkk', Crt)
# 
#     return E, nu, Gm, B, K
